@@ -18,8 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef _tds_h_
-#define _tds_h_
+#ifndef _tdsguard_hfOrWb5znoUCWdBPoNQvqN_
+#define _tdsguard_hfOrWb5znoUCWdBPoNQvqN_
 
 #ifndef _freetds_config_h_
 #error should include config.h before
@@ -61,6 +61,7 @@ typedef struct tds_bcpinfo TDSBCPINFO;
 #include <freetds/bool.h>
 #include <freetds/macros.h>
 #include <freetds/utils/string.h>
+#include <freetds/utils/path.h>
 #include <freetds/replacements.h>
 
 #include <freetds/pushvis.h>
@@ -81,9 +82,9 @@ extern "C"
 typedef struct tds_compiletime_settings
 {
 	const char *freetds_version;	/* release version of FreeTDS */
-	const char *sysconfdir;		/* location of freetds.conf */
+	const tds_dir_char *sysconfdir;		/* location of freetds.conf */
 	const char *last_update;	/* latest software_version date among the modules */
-	const char *tdsver;	/* TDS protocol version (4.2/4.6/5.0/7.0/7.1) 5.0 */
+	const char *tdsver;	/* TDS protocol version (4.2/4.6/5.0/7.0/7.1/8.0) 5.0 */
 	bool msdblib;		/* for MS style dblib */
 	bool sybase_compat;	/* enable increased Open Client binary compatibility */
 	bool threadsafe;		/* compile for thread safety default=no */
@@ -129,8 +130,6 @@ typedef int32_t TDS_INT;			/* 32-bit int      */
 typedef uint32_t TDS_UINT;			/* 32-bit unsigned */
 typedef int64_t TDS_INT8;			/* 64-bit integer  */
 typedef uint64_t TDS_UINT8;			/* 64-bit unsigned */
-typedef intptr_t TDS_INTPTR;
-typedef uintptr_t TDS_UINTPTR;
 typedef tds_sysdep_real32_type TDS_REAL;	/* 32-bit real     */
 typedef tds_sysdep_real64_type TDS_FLOAT;	/* 64-bit real     */
 
@@ -139,7 +138,7 @@ typedef tds_sysdep_real64_type TDS_FLOAT;	/* 64-bit real     */
 #define TDS_INVALID_TYPE ((TDS_SERVER_TYPE) 0)
 
 /**
- * this structure is not directed connected to a TDS protocol but
+ * This structure is not directly connected to TDS protocol but
  * keeps any DATE/TIME information.
  */
 typedef struct
@@ -243,30 +242,6 @@ enum tds_token_flags
 	TDS_TOKEN_TRAILING = TDS_STOPAT_ROWFMT|TDS_STOPAT_COMPUTEFMT|TDS_STOPAT_ROW|TDS_STOPAT_COMPUTE|TDS_STOPAT_MSG|TDS_STOPAT_OTHERS
 };
 
-/**
- * Flags returned in TDS_DONE token
- */
-enum tds_end
-{
-	  TDS_DONE_FINAL 	= 0x00	/**< final result set, command completed successfully. */
-	, TDS_DONE_MORE_RESULTS = 0x01	/**< more results follow */
-	, TDS_DONE_ERROR 	= 0x02	/**< error occurred */
-	, TDS_DONE_INXACT 	= 0x04	/**< transaction in progress */
-	, TDS_DONE_PROC 	= 0x08	/**< results are from a stored procedure */
-	, TDS_DONE_COUNT 	= 0x10	/**< count field in packet is valid */
-	, TDS_DONE_CANCELLED 	= 0x20	/**< acknowledging an attention command (usually a cancel) */
-	, TDS_DONE_EVENT 	= 0x40	/*   part of an event notification. */
-	, TDS_DONE_SRVERROR 	= 0x100	/**< SQL server server error */
-	
-	/* after the above flags, a TDS_DONE packet has a field describing the state of the transaction */
-	, TDS_DONE_NO_TRAN 	= 0	/* No transaction in effect */
-	, TDS_DONE_TRAN_SUCCEED = 1	/* Transaction completed successfully */
-	, TDS_DONE_TRAN_PROGRESS= 2	/* Transaction in progress */
-	, TDS_DONE_STMT_ABORT 	= 3	/* A statement aborted */
-	, TDS_DONE_TRAN_ABORT 	= 4	/* Transaction aborted */
-};
-
-
 /*
  * TDSERRNO is emitted by libtds to the client library's error handler
  * (which may in turn call the client's error handler).
@@ -338,7 +313,8 @@ typedef enum tds_encryption_level {
 	TDS_ENCRYPTION_DEFAULT,
 	TDS_ENCRYPTION_OFF,
 	TDS_ENCRYPTION_REQUEST,
-	TDS_ENCRYPTION_REQUIRE
+	TDS_ENCRYPTION_REQUIRE,
+	TDS_ENCRYPTION_STRICT
 } TDS_ENCRYPTION_LEVEL;
 
 /*
@@ -397,8 +373,8 @@ extern const char *const tds_type_names[256];
 #define is_binary_type(x)     ((tds_type_flags_ms[x] & TDS_TYPEFLAG_BINARY) != 0)
 #define is_char_type(x)       ((tds_type_flags_ms[x] & (TDS_TYPEFLAG_ASCII|TDS_TYPEFLAG_UNICODE)) != 0)
 #define is_similar_type(x, y) (is_char_type(x) && is_char_type(y))
-static inline
-bool is_tds_type_valid(int type)
+inline static bool
+is_tds_type_valid(int type)
 {
 	return (unsigned) type < 256u && tds_type_flags_ms[type] != 0;
 }
@@ -455,7 +431,9 @@ bool is_tds_type_valid(int type)
 #define TDS_STR_USE_UTF_16	"use utf-16"
 #define TDS_STR_LANGUAGE	"language"
 #define TDS_STR_APPENDMODE	"dump file append"
-#define TDS_STR_DATEFMT	"date format"
+#define TDS_STR_DATETIMEFMT	"date format"
+#define TDS_STR_DATEFMT	"date-only format"
+#define TDS_STR_TIMEFMT	"time-only format"
 #define TDS_STR_INSTANCE "instance"
 #define TDS_STR_ASA_DATABASE	"asa database"
 #define TDS_STR_DATABASE	"database"
@@ -466,6 +444,7 @@ bool is_tds_type_valid(int type)
 #define TDS_STR_ENCRYPTION_OFF	 "off"
 #define TDS_STR_ENCRYPTION_REQUEST "request"
 #define TDS_STR_ENCRYPTION_REQUIRE "require"
+#define TDS_STR_ENCRYPTION_STRICT "strict"
 /* Defines to enable optional GSSAPI delegation */
 #define TDS_GSSAPI_DELEGATION "enable gssapi delegation"
 /* Mutual authentication */
@@ -480,6 +459,8 @@ bool is_tds_type_valid(int type)
 #define TDS_STR_CRLFILE	"crl file"
 /* check SSL hostname */
 #define TDS_STR_CHECKSSLHOSTNAME	"check certificate hostname"
+/* SSL hostname to check certificate */
+#define TDS_STR_SSLHOSTNAME	"certificate hostname"
 /* database filename to attach on login (MSSQL) */
 #define TDS_STR_DBFILENAME	"database filename"
 /* Application Intent MSSQL 2012 support */
@@ -529,6 +510,7 @@ typedef struct tds_login
 	DSTR db_filename;		/**< database filename to attach (MSSQL) */
 	DSTR cafile;			/**< certificate authorities file */
 	DSTR crlfile;			/**< certificate revocation file */
+	DSTR certificate_host_name;	/**< certificate hostname to check, if empty use server_host_name */
 	DSTR openssl_ciphers;
 	DSTR app_name;
 	DSTR user_name;	    	/**< account for login */
@@ -545,7 +527,7 @@ typedef struct tds_login
 
 	struct addrinfo *ip_addrs;	  		/**< ip(s) of server */
 	DSTR instance_name;
-	DSTR dump_file;
+	tds_dir_char *dump_file;
 	int debug_flags;
 	int text_size;
 	DSTR routing_address;
@@ -567,6 +549,7 @@ typedef struct tds_login
 	unsigned int check_ssl_hostname:1;
 	unsigned int readonly_intent:1;
 	unsigned int enable_tls_v1:1;
+	unsigned int enable_tls_v1_specified:1;
 	unsigned int server_is_valid:1;
 } TDSLOGIN;
 
@@ -582,7 +565,9 @@ typedef struct tds_locale
 {
 	char *language;
 	char *server_charset;
+	char *datetime_fmt;
 	char *date_fmt;
+	char *time_fmt;
 } TDSLOCALE;
 
 /** 
@@ -594,11 +579,11 @@ typedef struct tds_blob
 	TDS_CHAR *textvalue;
 	TDS_CHAR textptr[16];
 	TDS_CHAR timestamp[8];
-	unsigned char valid_ptr;
+	bool valid_ptr;
 } TDSBLOB;
 
 /**
- * Store variant informations
+ * Store variant information
  */
 typedef struct tds_variant
 {
@@ -739,7 +724,7 @@ struct tds_column
 	/* FIXME find a best place to store these data, some are unused */
 	TDS_SMALLINT column_bindtype;
 	TDS_SMALLINT column_bindfmt;
-	TDS_UINT column_bindlen;
+	TDS_INT column_bindlen;
 	TDS_SMALLINT *column_nullbind;
 	TDS_CHAR *column_varaddr;
 	TDS_INT *column_lenbind;
@@ -799,7 +784,7 @@ typedef enum tds_operations
 {
 	TDS_OP_NONE		= 0,
 
-	/* mssql operations */
+	/* MSSQL operations, these matches protocol definitions */
 	TDS_OP_CURSOR		= TDS_SP_CURSOR,
 	TDS_OP_CURSOROPEN	= TDS_SP_CURSOROPEN,
 	TDS_OP_CURSORPREPARE	= TDS_SP_CURSORPREPARE,
@@ -931,7 +916,7 @@ typedef enum tds_cursor_fetch
 } TDS_CURSOR_FETCH;
 
 /**
- * Holds informations about a cursor
+ * Holds information about a cursor
  */
 typedef struct tds_cursor
 {
@@ -990,7 +975,7 @@ typedef struct tds_dynamic
 	 * This can happen for instance is tds protocol doesn't support dynamics or trying
 	 * to prepare query under Sybase that have BLOBs as parameters.
 	 */
-	TDS_TINYINT emulated;
+	bool emulated;
 	/**
 	 * true if dynamic was marker to be closed when connection is idle
 	 */
@@ -1024,6 +1009,7 @@ typedef struct tds_multiple
 /* forward declaration */
 typedef struct tds_context TDSCONTEXT;
 typedef int (*err_handler_t) (const TDSCONTEXT *, TDSSOCKET *, TDSMESSAGE *);
+typedef int (*int_handler_t) (void *);
 
 struct tds_context
 {
@@ -1090,6 +1076,7 @@ typedef struct tds_poll_wakeup
 struct tds_connection
 {
 	TDS_USMALLINT tds_version;
+	bool corked;
 	TDS_UINT product_version;	/**< version of product (Sybase/MS and full version) */
 	char *product_name;
 
@@ -1144,6 +1131,19 @@ struct tds_connection
 	int spid;
 	int client_spid;
 
+	/**
+	 * Ratio between bytes allocated for a NCHAR type and type length (Sybase).
+	 * For instance in case a NVARCHAR(3) takes 9 bytes it's 3.
+	 */
+	uint8_t ncharsize;
+
+	/**
+	 * Ratio between bytes allocated for a UNICHAR type and type length (Sybase).
+	 * For instance in case a UNIVARCHAR(2) takes 4 bytes it's 2.
+	 * It really should be only 2.
+	 */
+	uint8_t unicharsize;
+
 	void *tls_session;
 #if defined(HAVE_GNUTLS)
 	void *tls_credentials;
@@ -1271,20 +1271,33 @@ struct tds_socket
 #define tds_get_s(tds) ((tds)->conn->s)
 #define tds_set_s(tds, val) do { ((tds)->conn->s) = (val); } while(0)
 
+typedef struct tds_tvp_row
+{
+	TDSPARAMINFO *params;
+	struct tds_tvp_row *next;
+} TDS_TVP_ROW;
+
+typedef struct tds_tvp
+{
+	char *schema;
+	char *name;
+	TDSPARAMINFO *metadata;
+	TDS_TVP_ROW *row;
+} TDS_TVP;
+
 
 /* config.c */
 const TDS_COMPILETIME_SETTINGS *tds_get_compiletime_settings(void);
-typedef void (*TDSCONFPARSE) (const char *option, const char *value, void *param);
+typedef bool (*TDSCONFPARSE) (const char *option, const char *value, void *param);
 bool tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_parse, void *parse_param);
 bool tds_read_conf_file(TDSLOGIN * login, const char *server);
-void tds_parse_conf_section(const char *option, const char *value, void *param);
+bool tds_parse_conf_section(const char *option, const char *value, void *param);
 TDSLOGIN *tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale);
 void tds_fix_login(TDSLOGIN* login);
 TDS_USMALLINT * tds_config_verstr(const char *tdsver, TDSLOGIN* login);
 struct addrinfo *tds_lookup_host(const char *servername);
 TDSRET tds_lookup_host_set(const char *servername, struct addrinfo **addr);
 const char *tds_addrinfo2str(struct addrinfo *addr, char *name, int namemax);
-char *tds_get_home_file(const char *file);
 
 TDSRET tds_set_interfaces_file_loc(const char *interfloc);
 extern const char STD_DATETIME_FMT[];
@@ -1299,6 +1312,7 @@ TDSDYNAMIC *tds_lookup_dynamic(TDSCONNECTION * conn, const char *id);
 /*@observer@*/ const char *tds_prtype(int token);
 int tds_get_varint_size(TDSCONNECTION * conn, int datatype);
 TDS_SERVER_TYPE tds_get_cardinal_type(TDS_SERVER_TYPE datatype, int usertype);
+TDSRET tds8_adjust_login(TDSLOGIN *login);
 
 
 /* iconv.c */
@@ -1328,8 +1342,8 @@ void tds_free_context(TDSCONTEXT * locale);
 TDSPARAMINFO *tds_alloc_param_result(TDSPARAMINFO * old_param);
 void tds_free_input_params(TDSDYNAMIC * dyn);
 void tds_release_dynamic(TDSDYNAMIC ** dyn);
-static inline
-void tds_release_cur_dyn(TDSSOCKET * tds)
+inline static void
+tds_release_cur_dyn(TDSSOCKET * tds)
 {
 	tds_release_dynamic(&tds->cur_dyn);
 }
@@ -1338,7 +1352,7 @@ void tds_set_cur_dyn(TDSSOCKET *tds, TDSDYNAMIC *dyn);
 TDSSOCKET *tds_realloc_socket(TDSSOCKET * tds, size_t bufsize);
 char *tds_alloc_client_sqlstate(int msgno);
 char *tds_alloc_lookup_sqlstate(TDSSOCKET * tds, int msgno);
-TDSLOGIN *tds_alloc_login(int use_environment);
+TDSLOGIN *tds_alloc_login(bool use_environment);
 TDSDYNAMIC *tds_alloc_dynamic(TDSCONNECTION * conn, const char *id);
 void tds_free_login(TDSLOGIN * login);
 TDSLOGIN *tds_init_login(TDSLOGIN * login, TDSLOCALE * locale);
@@ -1354,8 +1368,6 @@ void tds_detach_results(TDSRESULTINFO *info);
 void * tds_realloc(void **pp, size_t new_size);
 #define TDS_RESIZE(p, n_elem) \
 	tds_realloc((void **) &(p), sizeof(*(p)) * (size_t) (n_elem))
-#define tds_new(type, n) ((type *) malloc(sizeof(type) * (n)))
-#define tds_new0(type, n) ((type *) calloc(n, sizeof(type)))
 
 TDSPACKET *tds_alloc_packet(void *buf, unsigned len);
 TDSPACKET *tds_realloc_packet(TDSPACKET *packet, unsigned len);
@@ -1363,6 +1375,7 @@ void tds_free_packets(TDSPACKET *packet);
 TDSBCPINFO *tds_alloc_bcpinfo(void);
 void tds_free_bcpinfo(TDSBCPINFO *bcpinfo);
 void tds_deinit_bcpinfo(TDSBCPINFO *bcpinfo);
+void tds_deinit_tvp(TDS_TVP *table);
 
 
 /* login.c */
@@ -1378,7 +1391,7 @@ bool tds_set_server(TDSLOGIN * tds_login, const char *server) TDS_WUR;
 bool tds_set_client_charset(TDSLOGIN * tds_login, const char *charset) TDS_WUR;
 bool tds_set_language(TDSLOGIN * tds_login, const char *language) TDS_WUR;
 void tds_set_version(TDSLOGIN * tds_login, TDS_TINYINT major_ver, TDS_TINYINT minor_ver);
-int tds_connect_and_login(TDSSOCKET * tds, TDSLOGIN * login);
+TDSRET tds_connect_and_login(TDSSOCKET * tds, TDSLOGIN * login);
 
 
 /* query.c */
@@ -1400,8 +1413,8 @@ TDSRET tds_submit_unprepare(TDSSOCKET * tds, TDSDYNAMIC * dyn);
 TDSRET tds_submit_rpc(TDSSOCKET * tds, const char *rpc_name, TDSPARAMINFO * params, TDSHEADERS * head);
 TDSRET tds_submit_optioncmd(TDSSOCKET * tds, TDS_OPTION_CMD command, TDS_OPTION option, TDS_OPTION_ARG *param, TDS_INT param_size);
 TDSRET tds_submit_begin_tran(TDSSOCKET *tds);
-TDSRET tds_submit_rollback(TDSSOCKET *tds, int cont);
-TDSRET tds_submit_commit(TDSSOCKET *tds, int cont);
+TDSRET tds_submit_rollback(TDSSOCKET *tds, bool cont);
+TDSRET tds_submit_commit(TDSSOCKET *tds, bool cont);
 TDSRET tds_disconnect(TDSSOCKET * tds);
 size_t tds_quote_id(TDSSOCKET * tds, char *buffer, const char *id, int idlen);
 size_t tds_quote_id_rpc(TDSSOCKET * tds, char *buffer, const char *id, int idlen);
@@ -1417,9 +1430,9 @@ void tds_convert_string_free(const char *original, const char *converted);
 #endif
 TDSRET tds_get_column_declaration(TDSSOCKET * tds, TDSCOLUMN * curcol, char *out);
 
-TDSRET tds_cursor_declare(TDSSOCKET * tds, TDSCURSOR * cursor, TDSPARAMINFO *params, int *send);
-TDSRET tds_cursor_setrows(TDSSOCKET * tds, TDSCURSOR * cursor, int *send);
-TDSRET tds_cursor_open(TDSSOCKET * tds, TDSCURSOR * cursor, TDSPARAMINFO *params, int *send);
+TDSRET tds_cursor_declare(TDSSOCKET * tds, TDSCURSOR * cursor, bool *send);
+TDSRET tds_cursor_setrows(TDSSOCKET * tds, TDSCURSOR * cursor, bool *send);
+TDSRET tds_cursor_open(TDSSOCKET * tds, TDSCURSOR * cursor, TDSPARAMINFO *params, bool *send);
 TDSRET tds_cursor_fetch(TDSSOCKET * tds, TDSCURSOR * cursor, TDS_CURSOR_FETCH fetch_type, TDS_INT i_row);
 TDSRET tds_cursor_get_cursor_info(TDSSOCKET * tds, TDSCURSOR * cursor, TDS_UINT * row_number, TDS_UINT * row_count);
 TDSRET tds_cursor_close(TDSSOCKET * tds, TDSCURSOR * cursor);
@@ -1436,7 +1449,6 @@ TDSRET tds_multiple_execute(TDSSOCKET *tds, TDSMULTIPLE *multiple, TDSDYNAMIC *
 
 /* token.c */
 TDSRET tds_process_cancel(TDSSOCKET * tds);
-int tds_get_token_size(int marker);
 TDSRET tds_process_login_tokens(TDSSOCKET * tds);
 TDSRET tds_process_simple_query(TDSSOCKET * tds);
 int tds5_send_optioncmd(TDSSOCKET * tds, TDS_OPTION_CMD tds_command, TDS_OPTION tds_option, TDS_OPTION_ARG * tds_argument,
@@ -1494,7 +1506,6 @@ int tdserror (const TDSCONTEXT * tds_ctx, TDSSOCKET * tds, int msgno, int errnum
 TDS_STATE tds_set_state(TDSSOCKET * tds, TDS_STATE state);
 void tds_swap_bytes(void *buf, int bytes);
 unsigned int tds_gettime_ms(void);
-char *tds_strndup(const void *s, TDS_INTPTR len);
 
 
 /* log.c */
@@ -1506,7 +1517,7 @@ void tdsdump_off(TDSDUMP_OFF_ITEM *off_item);
 void tdsdump_on(TDSDUMP_OFF_ITEM *off_item);
 int tdsdump_isopen(void);
 #include <freetds/popvis.h>
-int tdsdump_open(const char *filename);
+int tdsdump_open(const tds_dir_char *filename);
 #include <freetds/pushvis.h>
 void tdsdump_close(void);
 void tdsdump_dump_buf(const char* file, unsigned int level_line, const char *msg, const void *buf, size_t length);
@@ -1526,9 +1537,9 @@ void tdsdump_log(const char* file, unsigned int level_line, const char *fmt, ...
 #define TDSDUMP_BUF_FAST if (TDS_UNLIKELY(tds_write_dump)) tdsdump_dump_buf
 #define tdsdump_dump_buf TDSDUMP_BUF_FAST
 
-extern int tds_write_dump;
+extern bool tds_write_dump;
 extern int tds_debug_flags;
-extern int tds_g_append_mode;
+extern int tds_append_mode;
 
 
 /* net.c */
@@ -1540,18 +1551,20 @@ char *tds_prwsaerror(int erc);
 void tds_prwsaerror_free(char *s);
 int tds_connection_read(TDSSOCKET * tds, unsigned char *buf, int buflen);
 int tds_connection_write(TDSSOCKET *tds, const unsigned char *buf, int buflen, int final);
+void tds_connection_coalesce(TDSSOCKET *tds);
+void tds_connection_flush(TDSSOCKET *tds);
 #define TDSSELREAD  POLLIN
 #define TDSSELWRITE POLLOUT
 int tds_select(TDSSOCKET * tds, unsigned tds_sel, int timeout_seconds);
 void tds_connection_close(TDSCONNECTION *conn);
 int tds_goodread(TDSSOCKET * tds, unsigned char *buf, int buflen);
 int tds_goodwrite(TDSSOCKET * tds, const unsigned char *buffer, size_t buflen);
-void tds_socket_flush(TDS_SYS_SOCKET sock);
 int tds_socket_set_nonblocking(TDS_SYS_SOCKET sock);
 int tds_wakeup_init(TDSPOLLWAKEUP *wakeup);
 void tds_wakeup_close(TDSPOLLWAKEUP *wakeup);
 void tds_wakeup_send(TDSPOLLWAKEUP *wakeup, char cancel);
-static inline TDS_SYS_SOCKET tds_wakeup_get_fd(const TDSPOLLWAKEUP *wakeup)
+inline static TDS_SYS_SOCKET
+tds_wakeup_get_fd(const TDSPOLLWAKEUP *wakeup)
 {
 	return wakeup->s_signaled;
 }
@@ -1583,9 +1596,10 @@ void tds_freeze(TDSSOCKET *tds, TDSFREEZE *freeze, unsigned size_len);
 size_t tds_freeze_written(TDSFREEZE *freeze);
 TDSRET tds_freeze_abort(TDSFREEZE *freeze);
 TDSRET tds_freeze_close(TDSFREEZE *freeze);
+TDSRET tds_freeze_close_string(TDSFREEZE *freeze);
 TDSRET tds_freeze_close_len(TDSFREEZE *freeze, int32_t size);
 
-static void inline
+inline static void
 tds_set_current_send_packet(TDSSOCKET *tds, TDSPACKET *pkt)
 {
 	tds->send_packet = pkt;
@@ -1603,6 +1617,7 @@ tds_set_current_send_packet(TDSSOCKET *tds, TDSPACKET *pkt)
 	TDSFREEZE current_freeze[1]; \
 	tds_freeze((tds_socket), current_freeze, (len)); do { do
 #define TDS_END_LEN while(0); } while(tds_freeze_close(current_freeze), 0); } while(0);
+#define TDS_END_LEN_STRING while(0); } while(tds_freeze_close_string(current_freeze), 0); } while(0);
 
 #define TDS_START_LEN_TINYINT(tds_socket) TDS_START_LEN_GENERIC(tds_socket, 1)
 #define TDS_START_LEN_USMALLINT(tds_socket) TDS_START_LEN_GENERIC(tds_socket, 2)
@@ -1639,8 +1654,8 @@ void tds_random_buffer(unsigned char *out, int len);
 
 /* sec_negotiate.c */
 TDSAUTHENTICATION * tds5_negotiate_get_auth(TDSSOCKET * tds);
-static inline
-void tds5_negotiate_set_msg_type(TDSAUTHENTICATION * tds_auth, unsigned msg_type)
+inline static void
+tds5_negotiate_set_msg_type(TDSAUTHENTICATION * tds_auth, unsigned msg_type)
 {
 	if (tds_auth)
 		tds_auth->msg_type = msg_type;
@@ -1657,17 +1672,27 @@ enum tds_bcp_directions
 	TDS_BCP_QUERYOUT = 3
 };
 
+typedef struct tds5_colinfo
+{
+	TDS_TINYINT type;
+	TDS_TINYINT status;
+	TDS_SMALLINT offset;
+	TDS_INT length;
+} TDS5COLINFO;
+
 struct tds_bcpinfo
 {
-	const char *hint;
 	void *parent;
+	DSTR hint;
 	DSTR tablename;
 	TDS_CHAR *insert_stmt;
 	TDS_INT direction;
-	TDS_INT identity_insert_on;
-	TDS_INT xfer_init;
+	bool identity_insert_on;
+	bool xfer_init;
 	TDS_INT bind_count;
 	TDSRESULTINFO *bindinfo;
+	TDS5COLINFO *sybase_colinfo;
+	TDS_INT sybase_count;
 };
 
 TDSRET tds_bcp_init(TDSSOCKET *tds, TDSBCPINFO *bcpinfo);
@@ -1686,8 +1711,8 @@ TDSRET tds_writetext_continue(TDSSOCKET *tds, const TDS_UCHAR *text, TDS_UINT si
 TDSRET tds_writetext_end(TDSSOCKET *tds);
 
 
-static inline
-bool tds_capability_enabled(const TDS_CAPABILITY_TYPE *cap, unsigned cap_num)
+inline static bool
+tds_capability_enabled(const TDS_CAPABILITY_TYPE *cap, unsigned cap_num)
 {
 	return (cap->values[sizeof(cap->values)-1-(cap_num>>3)] >> (cap_num&7)) & 1;
 }
@@ -1710,15 +1735,16 @@ bool tds_capability_enabled(const TDS_CAPABILITY_TYPE *cap, unsigned cap_num)
 #define IS_TDS72_PLUS(x) ((x)->tds_version>=0x702)
 #define IS_TDS73_PLUS(x) ((x)->tds_version>=0x703)
 #define IS_TDS74_PLUS(x) ((x)->tds_version>=0x704)
+#define IS_TDS80_PLUS(x) ((x)->tds_version>=0x800)
 
 #define TDS_MAJOR(x) ((x)->tds_version >> 8)
 #define TDS_MINOR(x) ((x)->tds_version & 0xff)
 
 #define IS_TDSDEAD(x) (((x) == NULL) || (x)->state == TDS_DEAD)
 
-/** Check if product is Sybase (such as Adaptive Server Enterrprice). x should be a TDSSOCKET*. */
+/** Check if product is Sybase (such as Adaptive Server Enterprise). x should be a TDSSOCKET*. */
 #define TDS_IS_SYBASE(x) (!((x)->conn->product_version & 0x80000000u))
-/** Check if product is Microsft SQL Server. x should be a TDSSOCKET*. */
+/** Check if product is Microsoft SQL Server. x should be a TDSSOCKET*. */
 #define TDS_IS_MSSQL(x) (((x)->conn->product_version & 0x80000000u)!=0)
 
 /** Calc a version number for mssql. Use with TDS_MS_VER(7,0,842).
@@ -1743,4 +1769,4 @@ bool tds_capability_enabled(const TDS_CAPABILITY_TYPE *cap, unsigned cap_num)
 #define TDS_PUT_SMALLINT(tds,v) tds_put_smallint((tds), ((TDS_SMALLINT)(v)))
 #define TDS_PUT_BYTE(tds,v) tds_put_byte((tds), ((unsigned char)(v)))
 
-#endif /* _tds_h_ */
+#endif /* _tdsguard_hfOrWb5znoUCWdBPoNQvqN_ */
