@@ -107,12 +107,7 @@ static const char *opt_col_term = "\t";
 static const char *opt_row_term = "\n";
 static const char *opt_default_db = NULL;
 
-static int do_query(TDSSOCKET * tds, char *buf, int opt_flags);
-static int get_opt_flags(char *s, int *opt_flags);
-static void populate_login(TDSLOGIN * login, int argc, char **argv);
 static int tsql_handle_message(const TDSCONTEXT * context, TDSSOCKET * tds, TDSMESSAGE * msg);
-static int tsql_handle_error  (const TDSCONTEXT * context, TDSSOCKET * tds, TDSMESSAGE * msg);
-static void slurp_input_file(char *fname, char **mybuf, size_t *bufsz, size_t *buflen, int *line);
 
 static char *
 tsql_readline(char *prompt)
@@ -161,7 +156,7 @@ tsql_readline(char *prompt)
 }
 
 static void
-tsql_add_history(const char *s)
+tsql_add_history(const char *s TDS_UNUSED)
 {
 #ifdef HAVE_READLINE
 	if (istty)
@@ -295,7 +290,7 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 			TDSMESSAGE msg;
 			gettimeofday(&stop, NULL);
 			sprintf(message, "Total time for processing %d rows: %ld msecs\n",
-				rows, (long) ((stop.tv_sec - start.tv_sec) * 1000) + ((stop.tv_usec - start.tv_usec) / 1000));
+				rows, (long) ((stop.tv_sec - start.tv_sec) * 1000 + (stop.tv_usec - start.tv_usec) / 1000));
 
 			memset(&msg, 0, sizeof(TDSMESSAGE));
 			msg.server = "tsql";
@@ -328,7 +323,7 @@ tsql_print_usage(const char *progname)
 		"-o options:\n"
 		"\tf\tDo not print footer\n"
 		"\th\tDo not print header\n"
-		"\tt\tPrint time informations\n"
+		"\tt\tPrint time information\n"
 		"\tv\tPrint TDS version\n"
 		"\tq\tQuiet\n\n"
 		"\tDelimiters can be multi-char strings appropriately escaped for your shell.\n"
@@ -490,7 +485,21 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 			break;
 		case 'C':
 			settings = tds_get_compiletime_settings();
-			printf("%s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n",
+			printf("%s\n"
+			       "%35s: %s\n"
+			       "%35s: %" tdsPRIdir "\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n"
+			       "%35s: %s\n",
 			       "Compile-time settings (established with the \"configure\" script)",
 			       "Version", settings->freetds_version,
 			       "freetds.conf directory", settings->sysconfdir,
@@ -507,6 +516,7 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 			       "OpenSSL", yes_no(settings->openssl),
 			       "GnuTLS", yes_no(settings->gnutls),
 			       "MARS", yes_no(settings->mars));
+			tds_free_login(login);
 			exit(0);
 			break;
 		default:
@@ -528,13 +538,16 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 
 	if ((global_opt_flags & OPT_INSTANCES) && hostname) {
 		struct addrinfo *addr;
-		char *filename = getenv("TDSDUMP");
+		tds_dir_char *filename = tds_dir_getenv(TDS_DIR("TDSDUMP"));
 
 		if (filename) {
-			if (asprintf(&filename, "%s.instances", filename) < 0)
+			size_t len = tds_dir_len(filename) + 12;
+			tds_dir_char *path = tds_new(tds_dir_char, len);
+			if (!path)
 				exit(1);
-			tdsdump_open(filename);
-			free(filename);
+			tds_dir_snprintf(path, len, TDS_DIR("%s.instances"), filename);
+			tdsdump_open(path);
+			free(path);
 		}
 		if ((addr = tds_lookup_host(hostname)) != NULL) {
 			tds7_get_instance_ports(stderr, addr);
@@ -637,7 +650,7 @@ out_of_memory:
 }
 
 static int
-tsql_handle_message(const TDSCONTEXT * context, TDSSOCKET * tds, TDSMESSAGE * msg)
+tsql_handle_message(const TDSCONTEXT * context TDS_UNUSED, TDSSOCKET * tds TDS_UNUSED, TDSMESSAGE * msg)
 {
 	if (msg->msgno == 0) {
 		fprintf(stderr, "%s\n", msg->message);
@@ -666,7 +679,7 @@ tsql_handle_message(const TDSCONTEXT * context, TDSSOCKET * tds, TDSMESSAGE * ms
 }
 
 static int	/* error from library, not message from server */
-tsql_handle_error(const TDSCONTEXT * context, TDSSOCKET * tds, TDSMESSAGE * msg)
+tsql_handle_error(const TDSCONTEXT * context TDS_UNUSED, TDSSOCKET * tds TDS_UNUSED, TDSMESSAGE * msg)
 {
 	fprintf(stderr, "Error %d (severity %d):\n\t%s\n", msg->msgno, msg->severity, msg->message);
 	if (0 != msg->oserr) {
@@ -718,7 +731,7 @@ print_instance_data(TDSLOGIN *login)
 
 #if defined(HAVE_ALARM) && !defined(_WIN32)
 static void
-count_alarm(int s)
+count_alarm(int s TDS_UNUSED)
 {
 	static int count = 0;
 	char buf[64];
@@ -750,7 +763,7 @@ main(int argc, char **argv)
 
 	istty = isatty(0);
 
-	if (INITSOCKET()) {
+	if (tds_socket_init()) {
 		fprintf(stderr, "Unable to initialize sockets\n");
 		return 1;
 	}
@@ -758,28 +771,29 @@ main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 
 	/* grab a login structure */
-	login = tds_alloc_login(1);
+	login = tds_alloc_login(true);
 	if (!login) {
 		fprintf(stderr, "login cannot be null\n");
 		return 1;
 	}
 
+	/* process all the command line args into the login structure */
+	populate_login(login, argc, argv);
+
 	context = tds_alloc_context(NULL);
 	if (!context) {
+		tds_free_login(login);
 		fprintf(stderr, "context cannot be null\n");
 		return 1;
 	}
-	if (context->locale && !context->locale->date_fmt) {
+	if (context->locale && !context->locale->datetime_fmt) {
 		/* set default in case there's no locale file */
-		context->locale->date_fmt = strdup(STD_DATETIME_FMT);
+		context->locale->datetime_fmt = strdup(STD_DATETIME_FMT);
 	}
 
 	context->msg_handler = tsql_handle_message;
 	context->err_handler = tsql_handle_error;
 
-	/* process all the command line args into the login structure */
-	populate_login(login, argc, argv);
-
 	/* Try to open a connection */
 	tds = tds_alloc_socket(context, 512);
 	assert(tds);
@@ -943,7 +957,7 @@ main(int argc, char **argv)
 	tds_free_socket(tds);
 	tds_free_login(login);
 	tds_free_context(context);
-	DONESOCKET();
+	tds_socket_done();
 
 	return 0;
 }
