@@ -86,7 +86,7 @@ static TDSRET _bcp_no_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int
 
 static int rtrim(char *, int);
 static int rtrim_u16(uint16_t *str, int len, uint16_t space);
-static STATUS _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error, bool skip);
+static STATUS _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, bool *row_error, bool skip);
 static int _bcp_readfmt_colinfo(DBPROCESS * dbproc, char *buf, BCP_HOSTCOLINFO * ci);
 static int _bcp_get_term_var(const BYTE * pdata, const BYTE * term, int term_len);
 
@@ -207,7 +207,7 @@ bcp_init(DBPROCESS * dbproc, const char *tblname, const char *hfile, const char
 
 	dbproc->bcpinfo->direction = direction;
 
-	dbproc->bcpinfo->xfer_init  = 0;
+	dbproc->bcpinfo->xfer_init = false;
 	dbproc->bcpinfo->bind_count = 0;
 
 	if (TDS_FAILED(tds_bcp_init(dbproc->tds_socket, dbproc->bcpinfo))) {
@@ -509,7 +509,9 @@ RETCODE
 bcp_colfmt_ps(DBPROCESS * dbproc, int host_colnum, int host_type,
 	      int host_prefixlen, DBINT host_collen, BYTE * host_term, int host_termlen, int table_colnum, DBTYPEINFO * typeinfo)
 {
-	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED: bcp_colfmt_ps(%p, %d, %d)\n", dbproc, host_colnum, host_type);
+	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED: bcp_colfmt_ps(%p, %d, %d, %d, %d, %p, %d, %d, %p)\n",
+		    dbproc, host_colnum, host_type, host_prefixlen, (int) host_collen,
+		    host_term, host_termlen, table_colnum, typeinfo);
 	CHECK_CONN(FAIL);
 	CHECK_PARAMETER(dbproc->bcpinfo, SYBEBCPI, FAIL);
 	
@@ -616,7 +618,7 @@ bcp_getbatchsize(DBPROCESS * dbproc)
  * \return SUCCEED or FAIL.
  * \sa 	bcp_control(), 
  * 	bcp_exec(), 
- * \todo Simplify.  Remove \a valuelen, and dbproc->bcpinfo->hint = strdup(hints[i])
+ * \todo Simplify.  Remove \a valuelen.
  */
 RETCODE
 bcp_options(DBPROCESS * dbproc, int option, BYTE * value, int valuelen)
@@ -642,7 +644,8 @@ bcp_options(DBPROCESS * dbproc, int option, BYTE * value, int valuelen)
 
 		for (i = 0; hints[i]; i++) {	/* look up hint */
 			if (strncasecmp((char *) value, hints[i], strlen(hints[i])) == 0) {
-				dbproc->bcpinfo->hint = hints[i];	/* safe: hints[i] is static constant, above */
+				if (!tds_dstr_copy(&dbproc->bcpinfo->hint, hints[i]))
+					return FAIL;
 				return SUCCEED;
 			}
 		}
@@ -1054,14 +1057,14 @@ static TDSRET
 _bcp_convert_in(DBPROCESS *dbproc, TDS_SERVER_TYPE srctype, const TDS_CHAR *src, TDS_UINT srclen,
 		TDS_SERVER_TYPE desttype, BCPCOLDATA *coldata)
 {
-	int variable = 1;
+	bool variable = true;
 	CONV_RESULT cr, *p_cr;
 	TDS_INT len;
 
 	coldata->is_null = false;
 
 	if (!is_variable_type(desttype)) {
-		variable = 0;
+		variable = false;
 		p_cr = (CONV_RESULT *) coldata->data;
 	} else {
 		p_cr = &cr;
@@ -1138,7 +1141,7 @@ rtrim_bcpcol(TDSCOLUMN *bcpcol)
  * \sa 	BCP_SETL(), bcp_batch(), bcp_bind(), bcp_colfmt(), bcp_colfmt_ps(), bcp_collen(), bcp_colptr(), bcp_columns(), bcp_control(), bcp_done(), bcp_exec(), bcp_getl(), bcp_init(), bcp_moretext(), bcp_options(), bcp_readfmt(), bcp_sendrow()
  */
 static STATUS
-_bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error, bool skip)
+_bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, bool *row_error, bool skip)
 {
 	int i;
 
@@ -1260,7 +1263,7 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error, bool ski
 			if (TDS_FAILED(conv_res)) {
 				tdsdump_log(TDS_DBG_FUNC, "col %d: error converting %ld bytes!\n",
 							(i+1), (long) collen);
-				*row_error = TRUE;
+				*row_error = true;
 				free(coldata);
 				dbperror(dbproc, SYBEBCOR, 0);
 				return FAIL;
@@ -1273,7 +1276,7 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error, bool ski
 
 			if (col_bytes > 0x7fffffffl) {
 				free(coldata);
-				*row_error = TRUE;
+				*row_error = true;
 				tdsdump_log(TDS_DBG_FUNC, "data from file is too large!\n");
 				dbperror(dbproc, SYBEBCOR, 0);
 				return FAIL;
@@ -1296,7 +1299,7 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error, bool ski
 
 			coldata = tds_new(TDS_CHAR, 1 + collen);
 			if (coldata == NULL) {
-				*row_error = TRUE;
+				*row_error = true;
 				dbperror(dbproc, SYBEMEM, errno);
 				return FAIL;
 			}
@@ -1338,7 +1341,7 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error, bool ski
 						     desttype, bcpcol->bcp_column_data);
 				if (TDS_FAILED(rc)) {
 					hostcol->column_error = HOST_COL_CONV_ERROR;
-					*row_error = 1;
+					*row_error = true;
 					tdsdump_log(TDS_DBG_FUNC, 
 						"_bcp_read_hostfile failed to convert %d bytes at offset 0x%" PRIx64 " in the data file.\n",
 						    collen, (TDS_INT8) col_start);
@@ -1352,7 +1355,7 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, int *row_error, bool ski
 					if (!bcpcol->column_nullable) {
 						/* too bad if the column is not nullable */
 						hostcol->column_error = HOST_COL_NULL_ERROR;
-						*row_error = 1;
+						*row_error = true;
 						dbperror(dbproc, SYBEBCNN, 0);
 					}
 				}
@@ -1402,7 +1405,7 @@ bcp_sendrow(DBPROCESS * dbproc)
 	 * The first time sendrow is called after bcp_init,
 	 * there is a certain amount of initialisation to be done.
 	 */
-	if (dbproc->bcpinfo->xfer_init == 0) {
+	if (!dbproc->bcpinfo->xfer_init) {
 
 		/* The start_copy function retrieves details of the table's columns */
 		if (TDS_FAILED(tds_bcp_start_copy_in(tds, dbproc->bcpinfo))) {
@@ -1410,7 +1413,7 @@ bcp_sendrow(DBPROCESS * dbproc)
 			return FAIL;
 		}
 
-		dbproc->bcpinfo->xfer_init = 1;
+		dbproc->bcpinfo->xfer_init = true;
 
 	}
 
@@ -1439,7 +1442,8 @@ _bcp_exec_in(DBPROCESS * dbproc, DBINT * rows_copied)
 	STATUS ret;
 
 	int i, row_of_hostfile, rows_written_so_far;
-	int row_error, row_error_count;
+	int row_error_count;
+	bool row_error;
 	offset_type row_start, row_end;
 	offset_type error_row_size;
 	const size_t chunk_size = 0x20000u;
@@ -1470,7 +1474,7 @@ _bcp_exec_in(DBPROCESS * dbproc, DBINT * rows_copied)
 		bool skip;
 
 		row_start = ftello(hostfile);
-		row_error = 0;
+		row_error = false;
 
 		row_of_hostfile++;
 
@@ -1532,7 +1536,7 @@ _bcp_exec_in(DBPROCESS * dbproc, DBINT * rows_copied)
 					}
 
 					if (fread(row_in_error, chunk, 1, hostfile) != 1) {
-						printf("BILL fread failed after fseek\n");
+						tdsdump_log(TDS_DBG_ERROR, "BILL fread failed after fseek\n");
 					}
 					count = (int)fwrite(row_in_error, chunk, 1, errfile);
 					if( (size_t)count < chunk ) {
@@ -2165,7 +2169,7 @@ bcp_bind(DBPROCESS * dbproc, BYTE * varaddr, int prefixlen, DBINT varlen,
 }
 
 static void
-_bcp_null_error(TDSBCPINFO *bcpinfo, int index, int offset)
+_bcp_null_error(TDSBCPINFO *bcpinfo, int index TDS_UNUSED, int offset TDS_UNUSED)
 {
 	DBPROCESS *dbproc = (DBPROCESS *) bcpinfo->parent;
 	dbperror(dbproc, SYBEBCNN, 0);
@@ -2182,7 +2186,7 @@ _bcp_null_error(TDSBCPINFO *bcpinfo, int index, int offset)
  * \sa 	_bcp_add_fixed_columns, _bcp_add_variable_columns, _bcp_send_bcp_record
  */
 static TDSRET
-_bcp_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset)
+_bcp_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset TDS_UNUSED)
 {
 	TDS_SERVER_TYPE coltype, desttype;
 	int collen;
@@ -2227,9 +2231,9 @@ _bcp_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset)
 		if (bindcol->column_bindlen == 0)
 			goto null_data;
 		if (collen)
-			collen = (int) ((bindcol->column_bindlen < (TDS_UINT)collen) ? bindcol->column_bindlen : (TDS_UINT)collen);
+			collen = (bindcol->column_bindlen < collen) ? bindcol->column_bindlen : collen;
 		else
-			collen = (int) bindcol->column_bindlen;
+			collen = bindcol->column_bindlen;
 	}
 
 	desttype = tds_get_conversion_type(bindcol->column_type, bindcol->column_size);
@@ -2259,6 +2263,7 @@ _bcp_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset)
 					    desttype, bindcol->bcp_column_data);
 	if (TDS_FAILED(rc))
 		return rc;
+	rtrim_bcpcol(bindcol);
 
 	return TDS_SUCCESS;
 
@@ -2273,7 +2278,7 @@ null_data:
  * are already on bcp_column_data
  */
 static TDSRET
-_bcp_no_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset)
+_bcp_no_get_col_data(TDSBCPINFO *bcpinfo TDS_UNUSED, TDSCOLUMN *bindcol TDS_UNUSED, int offset TDS_UNUSED)
 {
 	return TDS_SUCCESS;
 }
