@@ -20,14 +20,15 @@
 #include <odbcinst.h>
 #endif
 
-#ifndef _WIN32
+#include <odbcss.h>
 #include <freetds/sysdep_private.h>
-#else
-#define TDS_SDIR_SEPARATOR "\\"
-#endif
-
 #include <freetds/replacements.h>
 
+struct odbc_buf{
+	struct odbc_buf *next;
+	void *buf;
+};
+
 HENV odbc_env;
 HDBC odbc_conn;
 HSTMT odbc_stmt;
@@ -45,20 +46,20 @@ static int freetds_driver = -1;
 static int tds_version = -1;
 static char db_str_version[32];
 
-static int
+static bool
 check_lib(char *path, const char *file)
 {
-	int len = strlen(path);
+	size_t len = strlen(path);
 	FILE *f;
 
 	strcat(path, file);
 	f = fopen(path, "rb");
 	if (f) {
 		fclose(f);
-		return 1;
+		return true;
 	}
 	path[len] = 0;
-	return 0;
+	return false;
 }
 
 /* this should be extended with all possible systems... */
@@ -78,14 +79,14 @@ static const char *const search_driver[] = {
 int
 odbc_read_login_info(void)
 {
-	static const char *PWD = "../../../PWD";
+	static const char PWD[] = "../../../PWD";
 	FILE *in = NULL;
 	char line[512];
 	char *s1, *s2;
 	const char *const *search_p;
 	char path[1024];
 	int len;
-	int ini_override = 1;
+	bool ini_override = true;
 #if defined(_WIN32) && !defined(TDS_NO_DM)
 	UWORD old_config_mode;
 #endif
@@ -154,7 +155,7 @@ odbc_read_login_info(void)
 
 	s1 = getenv("TDSINIOVERRIDE");
 	if (s1 && atoi(s1) == 0)
-		ini_override = 0;
+		ini_override = false;
 
 #if !defined(_WIN32) || defined(TDS_NO_DM)
 	/* craft out odbc.ini, avoid to read wrong one */
@@ -173,10 +174,12 @@ odbc_read_login_info(void)
 #else
 	if (ini_override && SQLGetConfigMode(&old_config_mode)) {
 		ODBC_BUF *odbc_buf = NULL;
+		LPCTSTR server = (LPCTSTR) T(odbc_server);
+		LPCTSTR filename = (LPCTSTR) T("odbc.ini");
 		SQLSetConfigMode(ODBC_USER_DSN);
-		SQLWritePrivateProfileString(T(odbc_server), T("Driver"), T(odbc_driver), T("odbc.ini"));
-		SQLWritePrivateProfileString(T(odbc_server), T("Database"), T(odbc_database), T("odbc.ini"));
-		SQLWritePrivateProfileString(T(odbc_server), T("Servername"), T(odbc_server), T("odbc.ini"));
+		SQLWritePrivateProfileString(server, (LPCTSTR) T("Driver"), (void *) T(odbc_driver), filename);
+		SQLWritePrivateProfileString(server, (LPCTSTR) T("Database"), (LPCTSTR) T(odbc_database), filename);
+		SQLWritePrivateProfileString(server, (LPCTSTR) T("Servername"), (LPCTSTR) T(odbc_server), filename);
 		SQLSetConfigMode(old_config_mode);
 		ODBC_FREE();
 	}
@@ -275,8 +278,8 @@ odbc_connect(void)
 		char *params;
 		SQLSMALLINT len;
 
-		asprintf(&params, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;%s",
-			 odbc_server, odbc_user, odbc_password, odbc_database, odbc_conn_additional_params);
+		assert(asprintf(&params, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;%s",
+				odbc_server, odbc_user, odbc_password, odbc_database, odbc_conn_additional_params) >= 0);
 		assert(params);
 		CHKDriverConnect(NULL, T(params), SQL_NTS, (SQLTCHAR *) command, sizeof(command)/sizeof(SQLTCHAR),
 				 &len, SQL_DRIVER_NOPROMPT, "SI");
@@ -312,6 +315,7 @@ odbc_disconnect(void)
 		odbc_conn = SQL_NULL_HDBC;
 	}
 
+	ODBC_FREE();
 	if (odbc_env) {
 		SQLFreeEnv(odbc_env);
 		odbc_env = SQL_NULL_HENV;
@@ -337,7 +341,7 @@ odbc_command_with_result(HSTMT stmt, const char *command)
 }
 
 static int ms_db = -1;
-int
+bool
 odbc_db_is_microsoft(void)
 {
 	ODBC_BUF *odbc_buf = NULL;
@@ -353,10 +357,10 @@ odbc_db_is_microsoft(void)
 		ms_db = (strstr(C(buf), "microsoft") != NULL);
 	}
 	ODBC_FREE();
-	return ms_db;
+	return !!ms_db;
 }
 
-int
+bool
 odbc_driver_is_freetds(void)
 {
 	ODBC_BUF *odbc_buf = NULL;
@@ -372,7 +376,7 @@ odbc_driver_is_freetds(void)
 		freetds_driver = (strstr(C(buf), "tds") != NULL);
 	}
 	ODBC_FREE();
-	return freetds_driver;
+	return !!freetds_driver;
 }
 
 /* Detect protocol version using queries
@@ -476,14 +480,15 @@ odbc_tds_version(void)
 	return tds_version;
 }
 
-const char *odbc_db_version(void)
+const char *
+odbc_db_version(void)
 {
 	if (!db_str_version[0]) {
 		ODBC_BUF *odbc_buf = NULL;
 		SQLTCHAR buf[32];
 		SQLSMALLINT version_len;
 
-		CHKR(SQLGetInfo, (odbc_conn, SQL_DBMS_VER, buf, sizeof(buf), &version_len), "S");
+		CHKGetInfo(SQL_DBMS_VER, buf, sizeof(buf), &version_len, "S");
 		strcpy(db_str_version, C(buf));
 		ODBC_FREE();
 	}
@@ -491,7 +496,8 @@ const char *odbc_db_version(void)
 	return db_str_version;
 }
 
-unsigned int odbc_db_version_int(void)
+unsigned int
+odbc_db_version_int(void)
 {
 	unsigned int h, l;
 	if (sscanf(odbc_db_version(), "%u.%u.", &h, &l) != 2) {
@@ -757,7 +763,7 @@ odbc_buf_asprintf(ODBC_BUF** buf, const char *fmt, ...)
 	char *ret = NULL;
 
 	va_start(ap, fmt);
-	vasprintf(&ret, fmt, ap);
+	assert(vasprintf(&ret, fmt, ap) >= 0);
 	va_end(ap);
 
 	return (char *) odbc_buf_add(buf, ret);
@@ -1001,3 +1007,66 @@ struct odbc_lookup_int odbc_sql_c_types[] = {
 #undef TYPE
 	{ NULL, 0 }
 };
+
+struct odbc_lookup_int odbc_sql_types[] = {
+#define TYPE(s) { #s, s }
+	TYPE(SQL_CHAR),
+	TYPE(SQL_VARCHAR),
+	TYPE(SQL_LONGVARCHAR),
+	TYPE(SQL_WCHAR),
+	TYPE(SQL_WVARCHAR),
+	TYPE(SQL_WLONGVARCHAR),
+	TYPE(SQL_DECIMAL),
+	TYPE(SQL_NUMERIC),
+	TYPE(SQL_SMALLINT),
+	TYPE(SQL_INTEGER),
+	TYPE(SQL_REAL),
+	TYPE(SQL_FLOAT),
+	TYPE(SQL_DOUBLE),
+	TYPE(SQL_BIT),
+	TYPE(SQL_TINYINT),
+	TYPE(SQL_BIGINT),
+	TYPE(SQL_BINARY),
+	TYPE(SQL_VARBINARY),
+	TYPE(SQL_LONGVARBINARY),
+	TYPE(SQL_DATE),
+	TYPE(SQL_TIME),
+	TYPE(SQL_TIMESTAMP),
+	TYPE(SQL_TYPE_DATE),
+	TYPE(SQL_TYPE_TIME),
+	TYPE(SQL_TYPE_TIMESTAMP),
+	TYPE(SQL_DATETIME),
+	TYPE(SQL_SS_VARIANT),
+	TYPE(SQL_SS_UDT),
+	TYPE(SQL_SS_XML),
+	TYPE(SQL_SS_TABLE),
+	TYPE(SQL_SS_TIME2),
+	TYPE(SQL_SS_TIMESTAMPOFFSET),
+#ifdef SQL_GUID
+	TYPE(SQL_GUID),
+#endif
+#undef TYPE
+	{ NULL, 0 }
+};
+
+#ifdef _MSC_VER
+/* See https://learn.microsoft.com/en-us/cpp/preprocessor/warning?view=msvc-170 */
+#pragma warning(push)
+#pragma warning(disable:4996)
+#endif
+SQLRETURN
+SQLSetStmtOption_nowarning(SQLHSTMT hstmt, SQLSMALLINT option, SQLULEN param)
+{
+	return SQLSetStmtOption(hstmt, option, param);
+}
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
+
+void
+odbc_swap_stmts(SQLHSTMT *a, SQLHSTMT *b)
+{
+	SQLHSTMT tmp = *a;
+	*a = *b;
+	*b = tmp;
+}
