@@ -1,4 +1,3 @@
-#undef NDEBUG
 #include "common.h"
 #include <assert.h>
 
@@ -7,11 +6,8 @@
 	This test on Sybase should not raise an error
 */
 
-static char software_version[] = "$Id: peter.c,v 1.2 2011-07-12 10:16:59 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 int
-main(int argc, char *argv[])
+main(void)
 {
 	SQLSMALLINT num_params, cols;
 	SQLLEN count;
@@ -26,7 +22,7 @@ main(int argc, char *argv[])
 
 	CHKPrepare(T("SELECT * FROM #tester WHERE id = ?"), SQL_NTS, "S");
 
-	CHKR(SQLNumParams, (odbc_stmt, &num_params), "S");
+	CHKNumParams(&num_params, "S");
 	assert(num_params == 1);
 
 	id = 1;
