@@ -216,14 +216,14 @@ tds_save(TDSSAVECONTEXT *ctx, char type, TDSMESSAGE *msg)
 }
 
 static int
-tds_save_msg(const TDSCONTEXT *ctx, TDSSOCKET *tds, TDSMESSAGE *msg)
+tds_save_msg(const TDSCONTEXT *ctx, TDSSOCKET *tds TDS_UNUSED, TDSMESSAGE *msg)
 {
 	tds_save((TDSSAVECONTEXT *) ctx, 0, msg);
 	return 0;
 }
 
 static int
-tds_save_err(const TDSCONTEXT *ctx, TDSSOCKET *tds, TDSMESSAGE *msg)
+tds_save_err(const TDSCONTEXT *ctx, TDSSOCKET *tds TDS_UNUSED, TDSMESSAGE *msg)
 {
 	tds_save((TDSSAVECONTEXT *) ctx, 1, msg);
 	return TDS_INT_CANCEL;
@@ -304,16 +304,74 @@ free_save_context(TDSSAVECONTEXT *ctx)
 }
 
 /**
- * Retrieve and set @@spid
+ * Set @@spid based on column data
+ * \tds
+ * @param curcol  column with spid data.
+ */
+static TDSRET
+tds_set_spid(TDSSOCKET * tds, TDSCOLUMN *curcol)
+{
+	switch (tds_get_conversion_type(curcol->column_type, curcol->column_size)) {
+	case SYBINT2:
+		tds->conn->spid = *((TDS_USMALLINT *) curcol->column_data);
+		break;
+	case SYBINT4:
+		tds->conn->spid = *((TDS_UINT *) curcol->column_data);
+		break;
+	default:
+		return TDS_FAIL;
+	}
+	return TDS_SUCCESS;
+}
+
+/**
+ * Set ncharsize based on column data.
+ * \tds
+ * @param res_info  resultset to get data from.
+ */
+static TDSRET
+tds_set_nvc(TDSSOCKET * tds, TDSRESULTINFO *res_info)
+{
+	int charsize;
+
+	/* Compute the ratios, put some acceptance in order to avoid issues. */
+	/* The "3" constant came from the query issued (NVARCHAR(3)) */
+	charsize = res_info->columns[0]->on_server.column_size / 3;
+	if (charsize >= 1 && charsize <= 4)
+		tds->conn->ncharsize = (uint8_t) charsize;
+	return TDS_SUCCESS;
+}
+
+/**
+ * Set unicharsize based on column data.
+ * \tds
+ * @param res_info  resultset to get data from.
+ */
+static TDSRET
+tds_set_uvc(TDSSOCKET * tds, TDSRESULTINFO *res_info)
+{
+	int charsize;
+
+	/* Compute the ratios, put some acceptance in order to avoid issues. */
+	/* The "3" constant came from the query issued (UNIVARCHAR(3)) */
+	charsize = res_info->columns[0]->on_server.column_size / 3;
+	if (charsize >= 1 && charsize <= 4)
+		tds->conn->unicharsize = (uint8_t) charsize;
+	return TDS_SUCCESS;
+}
+
+/**
+ * Parse the results from login queries
  * \tds
  */
 static TDSRET
-tds_set_spid(TDSSOCKET * tds)
+tds_parse_login_results(TDSSOCKET * tds, bool ignore_errors)
 {
 	TDS_INT result_type;
 	TDS_INT done_flags;
 	TDSRET rc;
 	TDSCOLUMN *curcol;
+	bool last_required = false;
 
 	CHECK_TDS_EXTRA(tds);
 
@@ -321,26 +379,27 @@ tds_set_spid(TDSSOCKET * tds)
 
 		switch (result_type) {
 		case TDS_ROW_RESULT:
-			if (!tds->res_info)
+			if (!tds->res_info && tds->res_info->num_cols < 1)
 				return TDS_FAIL;
-			if (tds->res_info->num_cols != 1)
-				break;
 			curcol = tds->res_info->columns[0];
-			switch (tds_get_conversion_type(curcol->column_type, curcol->column_size)) {
-			case SYBINT2:
-				tds->conn->spid = *((TDS_USMALLINT *) curcol->column_data);
-				break;
-			case SYBINT4:
-				tds->conn->spid = *((TDS_UINT *) curcol->column_data);
-				break;
-			default:
-				return TDS_FAIL;
+			if (tds->res_info->num_cols == 1 && strcmp(tds_dstr_cstr(&curcol->column_name), "spid") == 0)
+				rc = tds_set_spid(tds, curcol);
+			if (tds->res_info->num_cols == 1 && strcmp(tds_dstr_cstr(&curcol->column_name), "nvc") == 0) {
+				rc = tds_set_nvc(tds, tds->res_info);
+				last_required = true;
 			}
+			if (tds->res_info->num_cols == 1 && strcmp(tds_dstr_cstr(&curcol->column_name), "uvc") == 0)
+				rc = tds_set_uvc(tds, tds->res_info);
+			TDS_PROPAGATE(rc);
 			break;
 
 		case TDS_DONE_RESULT:
-			if ((done_flags & TDS_DONE_ERROR) != 0)
+		case TDS_DONEPROC_RESULT:
+		case TDS_DONEINPROC_RESULT:
+			if ((done_flags & TDS_DONE_ERROR) != 0 && !ignore_errors)
 				return TDS_FAIL;
+			if (last_required)
+				ignore_errors = true;
 			break;
 		}
 	}
@@ -351,31 +410,71 @@ tds_set_spid(TDSSOCKET * tds)
 }
 
 static TDSRET
-tds_setup_connection(TDSSOCKET *tds, TDSLOGIN *login, bool set_db, bool set_spid)
+tds_process_single(TDSSOCKET *tds, char *query, bool ignore_errors)
+{
+	TDSRET erc;
+
+	if (!query[0])
+		return TDS_SUCCESS;
+
+	/* submit and parse results */
+	erc = tds_submit_query(tds, query);
+	if (TDS_SUCCEED(erc))
+		erc = tds_parse_login_results(tds, ignore_errors);
+
+	/* prepare next query */
+	query[0] = 0;
+
+	if (TDS_FAILED(erc))
+		free(query);
+
+	return erc;
+}
+
+#define process_single(ignore_errors) do { \
+	if (!single_query && TDS_FAILED(erc = tds_process_single(tds, str, ignore_errors))) \
+		return erc; \
+} while(0)
+
+static TDSRET
+tds_setup_connection(TDSSOCKET *tds, TDSLOGIN *login, bool set_db, bool single_query)
 {
 	TDSRET erc;
 	char *str;
 	int len;
+	const char *const product_name = (tds->conn->product_name != NULL ? tds->conn->product_name : "");
+	const bool is_sql_anywhere = (strcasecmp(product_name, "SQL Anywhere") == 0);
+	const bool is_openserver = (strcasecmp(product_name, "OpenServer") == 0);
 
-	len = 128 + tds_quote_id(tds, NULL, tds_dstr_cstr(&login->database),-1);
+	len = 192 + tds_quote_id(tds, NULL, tds_dstr_cstr(&login->database),-1);
 	if ((str = tds_new(char, len)) == NULL)
 		return TDS_FAIL;
 
 	str[0] = 0;
-	if (login->text_size) {
-		sprintf(str, "set textsize %d ", login->text_size);
-	}
-	if (set_spid && tds->conn->spid == -1) {
-		strcat(str, "select @@spid ");
-	}
+	if (login->text_size)
+		sprintf(str, "SET TEXTSIZE %d\n", login->text_size);
+	process_single(false);
+
+	if (tds->conn->spid == -1 && !is_openserver)
+		strcat(str, "SELECT @@spid spid\n");
+	process_single(true);
+
 	/* Select proper database if specified.
 	 * SQL Anywhere does not support multiple databases and USE statement
 	 * so don't send the request to avoid connection failures */
-	if (set_db && !tds_dstr_isempty(&login->database) &&
-	    (tds->conn->product_name == NULL || strcasecmp(tds->conn->product_name, "SQL Anywhere") != 0)) {
-		strcat(str, "use ");
+	if (set_db && !tds_dstr_isempty(&login->database) && !is_sql_anywhere) {
+		strcat(str, "USE ");
 		tds_quote_id(tds, strchr(str, 0), tds_dstr_cstr(&login->database), -1);
+		strcat(str, "\n");
 	}
+	process_single(false);
+
+	if (IS_TDS50(tds->conn) && !is_sql_anywhere && !is_openserver) {
+		strcat(str, "SELECT CONVERT(NVARCHAR(3), 'abc') nvc\n");
+		if (tds->conn->product_version >= TDS_SYB_VER(12, 0, 0))
+			strcat(str, "EXECUTE ('SELECT CONVERT(UNIVARCHAR(3), ''xyz'') uvc')\n");
+	}
+	process_single(true);
 
 	/* nothing to set, just return */
 	if (str[0] == 0) {
@@ -385,15 +484,9 @@ tds_setup_connection(TDSSOCKET *tds, TDSLOGIN *login, bool set_db, bool set_spid
 
 	erc = tds_submit_query(tds, str);
 	free(str);
-	if (TDS_FAILED(erc))
-		return erc;
+	TDS_PROPAGATE(erc);
 
-	if (set_spid && tds->conn->spid == -1)
-		erc = tds_set_spid(tds);
-	else
-		erc = tds_process_simple_query(tds);
-
-	return erc;
+	return tds_parse_login_results(tds, false);
 }
 
 /**
@@ -408,7 +501,7 @@ tds_setup_connection(TDSSOCKET *tds, TDSLOGIN *login, bool set_db, bool set_spid
  * 		- TDSEFCON: connect(2) succeeded, login packet not acknowledged.  
  *		- TDS_FAIL: connect(2) succeeded, login failed.  
  */
-static int
+static TDSRET
 tds_connect(TDSSOCKET * tds, TDSLOGIN * login, int *p_oserr)
 {
 	int erc = -TDSEFCON;
@@ -424,7 +517,7 @@ tds_connect(TDSSOCKET * tds, TDSLOGIN * login, int *p_oserr)
 	 * A major version of 0 means try to guess the TDS version. 
 	 * We try them in an order that should work. 
 	 */
-	const static TDS_USMALLINT versions[] =
+	static const TDS_USMALLINT versions[] =
 		{ 0x704
 		, 0x500
 		};
@@ -478,10 +571,10 @@ tds_connect(TDSSOCKET * tds, TDSLOGIN * login, int *p_oserr)
 	/*
 	 * If a dump file has been specified, start logging
 	 */
-	if (!tds_dstr_isempty(&login->dump_file) && !tdsdump_isopen()) {
+	if (login->dump_file != NULL && !tdsdump_isopen()) {
 		if (login->debug_flags)
 			tds_debug_flags = login->debug_flags;
-		tdsdump_open(tds_dstr_cstr(&login->dump_file));
+		tdsdump_open(login->dump_file);
 	}
 
 	tds->login = login;
@@ -492,7 +585,7 @@ tds_connect(TDSSOCKET * tds, TDSLOGIN * login, int *p_oserr)
 	if (tds->conn->char_convs[client2ucs2]->to.cd == (iconv_t) -1) {
 		if (!tds_dstr_isempty(&login->client_charset)) {
 			if (TDS_FAILED(tds_iconv_open(tds->conn, tds_dstr_cstr(&login->client_charset), login->use_utf16)))
-				return -TDSEMEM;
+				return -TDSEICONVAVAIL;
 		}
 	}
 
@@ -517,6 +610,7 @@ tds_connect(TDSSOCKET * tds, TDSLOGIN * login, int *p_oserr)
 	tds->conn->capabilities = login->capabilities;
 
 reroute:
+	tds_ssl_deinit(tds->conn);
 	erc = TDSEINTF;
 	orig_port = login->port;
 	for (addrs = login->ip_addrs; addrs != NULL; addrs = addrs->ai_next) {
@@ -598,7 +692,7 @@ reroute:
 	if (TDS_FAILED(erc) || TDS_FAILED(tds_process_login_tokens(tds))) {
 		tdsdump_log(TDS_DBG_ERROR, "login packet %s\n", TDS_SUCCEED(erc)? "accepted":"rejected");
 		tds_close_socket(tds);
-		tdserror(tds_get_ctx(tds), tds, TDSEFCON, 0); 	/* "Adaptive Server connection failed" */
+		tdserror(tds_get_ctx(tds), tds, TDSEFCON, 0); 	/* "TDS server connection failed" */
 		return -TDSEFCON;
 	}
 
@@ -666,31 +760,38 @@ reroute:
 #endif
 
 	erc = tds_setup_connection(tds, login, !db_selected, true);
-	/* try without asking @@spid, some servers do not support it */
-	if (TDS_FAILED(erc) && tds->conn->spid == -1)
-		erc = tds_setup_connection(tds, login, !db_selected, false);
+	/* try one query at a time, some servers do not support some queries */
 	if (TDS_FAILED(erc))
-		return erc;
+		erc = tds_setup_connection(tds, login, !db_selected, false);
+	TDS_PROPAGATE(erc);
 
 	tds->query_timeout = login->query_timeout;
 	tds->login = NULL;
 	return TDS_SUCCESS;
 }
 
-int
+TDSRET
 tds_connect_and_login(TDSSOCKET * tds, TDSLOGIN * login)
 {
 	int oserr = 0;
+
+	TDS_PROPAGATE(tds8_adjust_login(login));
+
 	return tds_connect(tds, login, &oserr);
 }
 
-static int
+static void
 tds_put_login_string(TDSSOCKET * tds, const char *buf, int n)
 {
 	const int buf_len = buf ? (int)strlen(buf) : 0;
-	return tds_put_buf(tds, (const unsigned char *) buf, n, buf_len);
+	tds_put_buf(tds, (const unsigned char *) buf, n, buf_len);
 }
 
+#define tds_put_login_string(tds, buf, n) do { \
+	TDS_COMPILE_CHECK(range, (n) > 0 && (n) < 256); \
+	tds_put_login_string(tds, buf, (n)); \
+} while(0)
+
 static TDSRET
 tds_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 {
@@ -983,9 +1084,7 @@ tds7_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 
 
 	/* initialize ouput buffer for strings */
-	rc = tds_dynamic_stream_init(&data_stream, &data, 0);
-	if (TDS_FAILED(rc))
-		return rc;
+	TDS_PROPAGATE(tds_dynamic_stream_init(&data_stream, &data, 0));
 
 #define SET_FIELD_DSTR(field, dstr, len_limit) do { \
 	data_fields[field].ptr = tds_dstr_cstr(&(dstr)); \
@@ -1027,6 +1126,7 @@ tds7_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 				return TDS_FAIL;
 			}
 		} else if (ext_len && field == &data_fields[EXTENSION]) {
+			/* reserve 4 bytes in the stream to put the extention offset */
 			if (data_stream.stream.write(&data_stream.stream, 4) != 4) {
 				free(data);
 				return TDS_FAIL;
@@ -1068,6 +1168,9 @@ tds7_send_login(TDSSOCKET * tds, const TDSLOGIN * login)
 	case 0x703:
 		tds7version = tds73Version;
 		break;
+	case 0x800:
+		/* for TDS 8.0 version should be ignored and ALPN used,
+		 * practically clients/servers usually set this to 7.4 */
 	case 0x704:
 		tds7version = tds74Version;
 		break;
@@ -1198,6 +1301,29 @@ tds7_crypt_pass(const unsigned char *clear_pass, size_t len, unsigned char *cryp
 		crypt_pass[i] = ((clear_pass[i] << 4) | (clear_pass[i] >> 4)) ^ 0xA5;
 }
 
+#if defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+static inline TDS_TINYINT
+tds7_get_encryption_byte(TDS_TINYINT encryption_level)
+{
+	switch (encryption_level) {
+	/* The observation working with Microsoft SQL Server is that
+	   OFF did not mean off, and you would end up with encryption
+	   turned on. Therefore when the freetds.conf says encrypt = off
+	   we really want no encryption, and claiming lack of support
+	   works for that. Note that the configuration default in this
+	   subroutine always been request due to code above that
+	   tests for TDS_ENCRYPTION_DEFAULT.
+	*/
+	case TDS_ENCRYPTION_OFF:
+	case TDS_ENCRYPTION_STRICT:
+		return TDS7_ENCRYPT_NOT_SUP;
+	case TDS_ENCRYPTION_REQUIRE:
+		return TDS7_ENCRYPT_ON;
+	}
+	return TDS7_ENCRYPT_OFF;
+}
+#endif
+
 static TDSRET
 tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 {
@@ -1207,6 +1333,7 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 	TDS_CHAR crypt_flag;
 	unsigned int start_pos = 21;
 	TDSRET ret;
+	bool mars_replied = false;
 
 #define START_POS 21
 #define UI16BE(n) ((n) >> 8), ((n) & 0xffu)
@@ -1254,6 +1381,10 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 	if (encryption_level == TDS_ENCRYPTION_DEFAULT)
 		encryption_level = TDS_ENCRYPTION_REQUEST;
 
+	/* all encrypted */
+	if (encryption_level == TDS_ENCRYPTION_STRICT)
+		TDS_PROPAGATE(tds_ssl_init(tds, true));
+
 	/*
 	 * fix a problem with mssql2k which doesn't like
 	 * packet splitted during SSL handshake
@@ -1272,17 +1403,7 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 	/* not supported */
 	tds_put_byte(tds, TDS7_ENCRYPT_NOT_SUP);
 #else
-	/* The observation working with Microsoft SQL Server is that
-	   OFF did not mean off, and you would end up with encryption
-	   turned on. Therefore when the freetds.conf says encrypt = off
-	   we really want no encryption, and claiming lack of support
-	   works for that. Note that the configuration default in this
-	   subroutine always been request due to code above that
-	   tests for TDS_ENCRYPTION_DEFAULT.
-	*/
-	tds_put_byte(tds, encryption_level == TDS_ENCRYPTION_OFF ? TDS7_ENCRYPT_NOT_SUP :
-			  encryption_level >= TDS_ENCRYPTION_REQUIRE ? TDS7_ENCRYPT_ON :
-			  TDS7_ENCRYPT_OFF);
+	tds_put_byte(tds, tds7_get_encryption_byte(encryption_level));
 #endif
 	/* instance */
 	tds_put_n(tds, instance_name, instance_name_len);
@@ -1296,9 +1417,7 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 #else
 		tds_put_byte(tds, 0);
 #endif
-	ret = tds_flush_packet(tds);
-	if (TDS_FAILED(ret))
-		return ret;
+	TDS_PROPAGATE(tds_flush_packet(tds));
 
 	/* now process reply from server */
 	ret = tds_read_packet(tds);
@@ -1330,10 +1449,12 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 		if (type == 1 && len >= 1) {
 			crypt_flag = p[off];
 		}
+		if (IS_TDS72_PLUS(tds->conn) && type == 4 && len >= 1) {
+			mars_replied = true;
 #if ENABLE_ODBC_MARS
-		if (IS_TDS72_PLUS(tds->conn) && type == 4 && len >= 1)
 			login->mars = p[off];
 #endif
+		}
 	}
 	/* we readed all packet */
 	tds->in_pos += pkt_len;
@@ -1341,10 +1462,13 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 
 	tdsdump_log(TDS_DBG_INFO1, "detected crypt flag %d\n", crypt_flag);
 
-	/* if server do not has certificate do normal login */
-	if (crypt_flag == TDS7_ENCRYPT_NOT_SUP) {
+	if (!mars_replied && encryption_level != TDS_ENCRYPTION_STRICT)
+		tds->conn->tds_version = 0x701;
+
+	/* if server does not have certificate or TLS already setup do normal login */
+	if (crypt_flag == TDS7_ENCRYPT_NOT_SUP || encryption_level == TDS_ENCRYPTION_STRICT) {
 		/* unless we wanted encryption and got none, then fail */
-		if (encryption_level >= TDS_ENCRYPTION_REQUIRE)
+		if (encryption_level == TDS_ENCRYPTION_REQUIRE)
 			return TDS_FAIL;
 
 		return tds7_send_login(tds, login);
@@ -1357,9 +1481,7 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 
 	/* here we have to do encryption ... */
 
-	ret = tds_ssl_init(tds);
-	if (TDS_FAILED(ret))
-		return ret;
+	TDS_PROPAGATE(tds_ssl_init(tds, false));
 
 	/* server just encrypt the first packet */
 	if (crypt_flag == TDS7_ENCRYPT_OFF)
