@@ -145,8 +145,10 @@ tds_convert_string(TDSSOCKET * tds, TDSICONV * char_conv, const char *s, int len
 	/* allocate needed buffer (+1 is to exclude 0 case) */
 	ol = il * char_conv->to.charset.max_bytes_per_char / char_conv->from.charset.min_bytes_per_char + 1;
 	buf = tds_new(char, ol);
-	if (!buf)
+	if (!buf) {
+		*out_len = 0;
 		return NULL;
+	}
 
 	ib = s;
 	ob = buf;
@@ -700,6 +702,26 @@ tds_count_placeholders_ucs2le(const char *query, const char *query_end)
 	}
 }
 
+static const char*
+tds50_char_declaration_from_usertype(TDSSOCKET *tds, TDS_INT usertype, unsigned int *p_size)
+{
+	switch (usertype) {
+	case USER_CHAR_TYPE:
+		return "CHAR(%u)";
+	case USER_VARCHAR_TYPE:
+		return "VARCHAR(%u)";
+	case USER_SYSNAME_TYPE:
+		return "SYSNAME";
+	case USER_NCHAR_TYPE:
+		*p_size /= tds->conn->ncharsize;
+		return "NCHAR(%u)";
+	case USER_NVARCHAR_TYPE:
+		*p_size /= tds->conn->ncharsize;
+		return "NVARCHAR(%u)";
+	}
+	return NULL;
+}
+
 /**
  * Return declaration for column (like "varchar(20)").
  *
@@ -729,10 +751,21 @@ tds_get_column_declaration(TDSSOCKET * tds, TDSCOLUMN * curcol, char *out)
 
 	switch (tds_get_conversion_type(curcol->on_server.column_type, curcol->on_server.column_size)) {
 	case XSYBCHAR:
+		if (IS_TDS50(tds->conn)) {
+			max_len = 32767;
+			fmt = tds50_char_declaration_from_usertype(tds, curcol->column_usertype, &size);
+			if (fmt != NULL)
+				break;
+		}
 	case SYBCHAR:
 		fmt = "CHAR(%u)";
 		break;
 	case SYBVARCHAR:
+		if (IS_TDS50(tds->conn)) {
+			fmt = tds50_char_declaration_from_usertype(tds, curcol->column_usertype, &size);
+			if (fmt != NULL)
+				break;
+		}
 	case XSYBVARCHAR:
 		if (curcol->column_varint_size == 8)
 			fmt = "VARCHAR(MAX)";
@@ -773,6 +806,25 @@ tds_get_column_declaration(TDSSOCKET * tds, TDSCOLUMN * curcol, char *out)
 		break;
 	case SYBLONGBINARY:	/* TODO correct ?? */
 	case SYBIMAGE:
+		if (IS_TDS50(tds->conn)) {
+			switch (curcol->column_usertype) {
+			case USER_UNICHAR_TYPE:
+				size /= 2u;
+				max_len = 8192;
+				fmt = "UNICHAR(%u)";
+				break;
+			case USER_UNIVARCHAR_TYPE:
+				size /= 2u;
+				max_len = 8192;
+				fmt = "UNIVARCHAR(%u)";
+				break;
+			case USER_UNITEXT_TYPE:
+				fmt = "UNITEXT";
+				break;
+			}
+			if (fmt != NULL)
+				break;
+		}
 		fmt = "IMAGE";
 		break;
 	case SYBMONEY4:
@@ -1153,7 +1205,7 @@ tds_submit_prepare(TDSSOCKET * tds, const char *query, const char *id, TDSDYNAMI
 	}
 
 	if (!IS_TDS50(tds->conn) && !IS_TDS7_PLUS(tds->conn)) {
-		dyn->emulated = 1;
+		dyn->emulated = true;
 		tds_dynamic_deallocated(tds->conn, dyn);
 		tds_set_state(tds, TDS_IDLE);
 		return TDS_SUCCESS;
@@ -1336,7 +1388,7 @@ tds_submit_execdirect(TDSSOCKET * tds, const char *query, TDSPARAMINFO * params,
 		if (!params) {
 			ret = tds_submit_query(tds, query);
 		} else {
-			dyn->emulated = 1;
+			dyn->emulated = true;
 			dyn->params = params;
 			dyn->query = strdup(query);
 			if (!dyn->query)
@@ -1488,7 +1540,7 @@ failure:
  * Get column size for wire
  */
 size_t
-tds_fix_column_size(TDSSOCKET * tds, TDSCOLUMN * curcol)
+tds_fix_column_size(TDSSOCKET * tds TDS_UNUSED, TDSCOLUMN * curcol)
 {
 	size_t size = curcol->on_server.column_size, min;
 
@@ -1542,15 +1594,11 @@ tds_put_data_info(TDSSOCKET * tds, TDSCOLUMN * curcol, int flags)
 		tdsdump_log(TDS_DBG_ERROR, "tds_put_data_info putting param_name \n");
 
 		if (IS_TDS7_PLUS(tds->conn)) {
-			TDSFREEZE outer;
-			size_t written;
-
-			tds_freeze(tds, &outer, 1);
-			if ((flags & TDS_PUT_DATA_PREFIX_NAME) != 0)
-				tds_put_smallint(tds, '@');
-			tds_put_string(tds, tds_dstr_cstr(&curcol->column_name), len);
-			written = (tds_freeze_written(&outer) - 1) / 2;
-			tds_freeze_close_len(&outer, written);
+			TDS_START_LEN_TINYINT(tds) { /* param name len */
+				if ((flags & TDS_PUT_DATA_PREFIX_NAME) != 0)
+					tds_put_smallint(tds, '@');
+				tds_put_string(tds, tds_dstr_cstr(&curcol->column_name), len);
+			} TDS_END_LEN_STRING
 		} else {
 			TDS_START_LEN_TINYINT(tds) { /* param name len */
 				tds_put_string(tds, tds_dstr_cstr(&curcol->column_name), len);
@@ -1923,13 +1971,13 @@ tds4_send_emulated_rpc(TDSSOCKET * tds, const char *rpc_name, TDSPARAMINFO * par
  * Calls a RPC from server. Output parameters will be stored in tds->param_info.
  * \param tds      state information for the socket and the TDS protocol
  * \param rpc_name name of RPC
- * \param params   parameters informations. NULL for no parameters
+ * \param params   parameters information. NULL for no parameters
  */
 TDSRET
 tds_submit_rpc(TDSSOCKET * tds, const char *rpc_name, TDSPARAMINFO * params, TDSHEADERS * head)
 {
 	TDSCOLUMN *param;
-	int rpc_name_len, i;
+	int i;
 	int num_params = params ? params->num_cols : 0;
 
 	CHECK_TDS_EXTRA(tds);
@@ -1945,19 +1993,14 @@ tds_submit_rpc(TDSSOCKET * tds, const char *rpc_name, TDSPARAMINFO * params, TDS
 	/* distinguish from dynamic query  */
 	tds_release_cur_dyn(tds);
 
-	rpc_name_len = (int)strlen(rpc_name);
 	if (IS_TDS7_PLUS(tds->conn)) {
-		TDSFREEZE outer;
-		size_t written;
-
 		if (tds_start_query_head(tds, TDS_RPC, head) != TDS_SUCCESS)
 			return TDS_FAIL;
 
 		/* procedure name */
-		tds_freeze(tds, &outer, 2);
-		tds_put_string(tds, rpc_name, rpc_name_len);
-		written = tds_freeze_written(&outer) / 2 - 1;
-		tds_freeze_close_len(&outer, written);
+		TDS_START_LEN_USMALLINT(tds) {
+			tds_put_string(tds, rpc_name, -1);
+		} TDS_END_LEN_STRING
 
 		/*
 		 * TODO support flags
@@ -1983,7 +2026,7 @@ tds_submit_rpc(TDSSOCKET * tds, const char *rpc_name, TDSPARAMINFO * params, TDS
 		tds_put_byte(tds, TDS_DBRPC_TOKEN);
 		TDS_START_LEN_USMALLINT(tds) {
 			TDS_START_LEN_TINYINT(tds) {
-				tds_put_string(tds, rpc_name, rpc_name_len);
+				tds_put_string(tds, rpc_name, -1);
 			} TDS_END_LEN
 			/* TODO flags */
 			tds_put_smallint(tds, num_params ? 2 : 0);
@@ -2116,7 +2159,6 @@ tds_send_cancel(TDSSOCKET * tds)
 
 /**
  * Quote a string properly. Output string is always NUL-terminated
- * \tds
  * \param buffer   output buffer. If NULL function will just return
  *        required bytes
  * \param quoting  quote character (should be one of '\'', '"', ']')
@@ -2125,14 +2167,12 @@ tds_send_cancel(TDSSOCKET * tds)
  * \returns size of output string
  */
 static size_t
-tds_quote(TDSSOCKET * tds, char *buffer, char quoting, const char *id, size_t len)
+tds_quote(char *buffer, char quoting, const char *id, size_t len)
 {
 	size_t size;
 	const char *src, *pend;
 	char *dst;
 
-	CHECK_TDS_EXTRA(tds);
-
 	pend = id + len;
 
 	/* quote */
@@ -2178,7 +2218,7 @@ tds_quote_id(TDSSOCKET * tds, char *buffer, const char *id, int idlen)
 
 	/* quote always for mssql */
 	if (TDS_IS_MSSQL(tds) || tds->conn->product_version >= TDS_SYB_VER(12, 5, 1))
-		return tds_quote(tds, buffer, ']', id, len);
+		return tds_quote(buffer, ']', id, len);
 
 	/* need quote ?? */
 	for (i = 0; i < len; ++i) {
@@ -2192,7 +2232,7 @@ tds_quote_id(TDSSOCKET * tds, char *buffer, const char *id, int idlen)
 			continue;
 		if (c == '_')
 			continue;
-		return tds_quote(tds, buffer, '\"', id, len);
+		return tds_quote(buffer, '\"', id, len);
 	}
 
 	if (buffer) {
@@ -2225,7 +2265,7 @@ tds_quote_id_rpc(TDSSOCKET * tds, char *buffer, const char *id, int idlen)
 
 	len = idlen < 0 ? strlen(id) : (size_t) idlen;
 
-	return tds_quote(tds, buffer, quote_id_char, id, len);
+	return tds_quote(buffer, quote_id_char, id, len);
 }
 
 /**
@@ -2238,9 +2278,9 @@ tds_quote_id_rpc(TDSSOCKET * tds, char *buffer, const char *id, int idlen)
  * \result written chars (not including needed terminator)
  */
 size_t
-tds_quote_string(TDSSOCKET * tds, char *buffer, const char *str, int len)
+tds_quote_string(TDSSOCKET * tds TDS_UNUSED, char *buffer, const char *str, int len)
 {
-	return tds_quote(tds, buffer, '\'', str, len < 0 ? strlen(str) : (size_t) len);
+	return tds_quote(buffer, '\'', str, len < 0 ? strlen(str) : (size_t) len);
 }
 
 /**
@@ -2259,7 +2299,7 @@ tds_set_cur_cursor(TDSSOCKET *tds, TDSCURSOR *cursor)
 }
 
 TDSRET
-tds_cursor_declare(TDSSOCKET * tds, TDSCURSOR * cursor, TDSPARAMINFO *params, int *something_to_send)
+tds_cursor_declare(TDSSOCKET * tds, TDSCURSOR * cursor, bool *something_to_send)
 {
 	CHECK_TDS_EXTRA(tds);
 
@@ -2298,14 +2338,14 @@ tds_cursor_declare(TDSSOCKET * tds, TDSCURSOR * cursor, TDSPARAMINFO *params, in
 			} TDS_END_LEN
 			tds_put_tinyint(tds, 0);	/* number of columns = 0 , valid value applicable only for updatable cursor */
 		} TDS_END_LEN
-		*something_to_send = 1;
+		*something_to_send = true;
 	}
 
 	return TDS_SUCCESS;
 }
 
 TDSRET
-tds_cursor_open(TDSSOCKET * tds, TDSCURSOR * cursor, TDSPARAMINFO *params, int *something_to_send)
+tds_cursor_open(TDSSOCKET * tds, TDSCURSOR * cursor, TDSPARAMINFO *params, bool *something_to_send)
 {
 	CHECK_TDS_EXTRA(tds);
 
@@ -2336,7 +2376,7 @@ tds_cursor_open(TDSSOCKET * tds, TDSCURSOR * cursor, TDSPARAMINFO *params, int *
 			} TDS_END_LEN
 			tds_put_byte(tds, 0);	/* Cursor status : 0 for no arguments */
 		} TDS_END_LEN
-		*something_to_send = 1;
+		*something_to_send = true;
 	}
 	if (IS_TDS7_PLUS(tds->conn)) {
 		const char *converted_query;
@@ -2437,7 +2477,7 @@ tds_cursor_open(TDSSOCKET * tds, TDSCURSOR * cursor, TDSPARAMINFO *params, int *
 		}
 		tds_freeze_close(&outer);
 
-		*something_to_send = 1;
+		*something_to_send = true;
 		tds->current_op = TDS_OP_CURSOROPEN;
 		tdsdump_log(TDS_DBG_ERROR, "tds_cursor_open (): RPC call set up \n");
 	}
@@ -2448,7 +2488,7 @@ tds_cursor_open(TDSSOCKET * tds, TDSCURSOR * cursor, TDSPARAMINFO *params, int *
 }
 
 TDSRET
-tds_cursor_setrows(TDSSOCKET * tds, TDSCURSOR * cursor, int *something_to_send)
+tds_cursor_setrows(TDSSOCKET * tds, TDSCURSOR * cursor, bool *something_to_send)
 {
 	CHECK_TDS_EXTRA(tds);
 
@@ -2490,7 +2530,7 @@ tds_cursor_setrows(TDSSOCKET * tds, TDSCURSOR * cursor, int *something_to_send)
 			tds_put_byte(tds, 0x20);	/* Status - TDS_CUR_ISTAT_ROWCNT 0x0020 */
 			tds_put_int(tds, cursor->cursor_rows);	/* row count to set */
 		} TDS_END_LEN
-		*something_to_send = 1;
+		*something_to_send = true;
 
 	}
 	return TDS_SUCCESS;
@@ -3690,7 +3730,7 @@ tds_submit_begin_tran(TDSSOCKET *tds)
  * \sa tds_submit_begin_tran, tds_submit_commit
  */
 TDSRET
-tds_submit_rollback(TDSSOCKET *tds, int cont)
+tds_submit_rollback(TDSSOCKET *tds, bool cont)
 {
 	CHECK_TDS_EXTRA(tds);
 
@@ -3721,7 +3761,7 @@ tds_submit_rollback(TDSSOCKET *tds, int cont)
  * \sa tds_submit_rollback, tds_submit_begin_tran
  */
 TDSRET
-tds_submit_commit(TDSSOCKET *tds, int cont)
+tds_submit_commit(TDSSOCKET *tds, bool cont)
 {
 	CHECK_TDS_EXTRA(tds);
 
