@@ -55,6 +55,7 @@
 
 #include <freetds/tds.h>
 #include <freetds/utils/string.h>
+#include <freetds/utils.h>
 #include <freetds/tls.h>
 #include <freetds/alloca.h>
 #include <freetds/replacements.h>
@@ -108,6 +109,16 @@ BIO_get_data(const BIO *b)
 #define ASN1_STRING_get0_data(x) ASN1_STRING_data(x)
 #endif
 
+#if ENABLE_ODBC_MARS
+#define CONN2TDS(conn) (conn->in_net_tds)
+#else
+#define CONN2TDS(conn) ((TDSSOCKET *) conn)
+#endif
+
+/* tds/8.0 */
+#define TDS8_ALPN_ARRAY 't', 'd', 's', '/', '8', '.', '0'
+#define TDS8_ALPN_ARRAY_LEN 7
+
 static SSL_RET
 tds_pull_func_login(SSL_PULL_ARGS)
 {
@@ -159,12 +170,8 @@ tds_pull_func(SSL_PULL_ARGS)
 
 	tdsdump_log(TDS_DBG_FUNC, "in tds_pull_func\n");
 
-#if ENABLE_ODBC_MARS
-	tds = conn->in_net_tds;
+	tds = CONN2TDS(conn);
 	assert(tds);
-#else
-	tds = (TDSSOCKET *) conn;
-#endif
 
 	/* already initialized (crypted TDS packets) */
 
@@ -183,17 +190,37 @@ tds_push_func(SSL_PUSH_ARGS)
 	tdsdump_log(TDS_DBG_FUNC, "in tds_push_func\n");
 
 	/* write to socket directly */
-#if ENABLE_ODBC_MARS
-	tds = conn->in_net_tds;
-#else
-	tds = (TDSSOCKET *) conn;
-#endif
+	tds = CONN2TDS(conn);
 	return tds_goodwrite(tds, (const unsigned char*) data, len);
 }
 
 static int tls_initialized = 0;
 static tds_mutex tls_mutex = TDS_MUTEX_INITIALIZER;
 
+#if ENABLE_ODBC_MARS
+static void
+set_current_tds(TDSCONNECTION *conn, TDSSOCKET *tds)
+{
+	tds_mutex_lock(&conn->list_mtx);
+	conn->in_net_tds = tds;
+	tds_mutex_unlock(&conn->list_mtx);
+}
+#else
+static inline void
+set_current_tds(TDSCONNECTION *conn TDS_UNUSED, TDSSOCKET *tds TDS_UNUSED)
+{
+}
+#endif
+
+static const char *
+wanted_certificate_hostname(TDSLOGIN *login)
+{
+	if (!tds_dstr_isempty(&login->certificate_host_name))
+		return tds_dstr_cstr(&login->certificate_host_name);
+
+	return tds_dstr_cstr(&login->server_host_name);
+}
+
 #ifdef HAVE_GNUTLS
 
 static void
@@ -356,19 +383,20 @@ tds_certificate_set_x509_system_trust(gnutls_certificate_credentials_t cred)
 #endif
 
 static int
-tds_verify_certificate(gnutls_session_t session)
+tds_verify_certificate(gnutls_session_t session, TDSSOCKET *tds)
 {
 	unsigned int status;
 	int ret;
-	TDSSOCKET *tds = (TDSSOCKET *) gnutls_transport_get_ptr(session);
 
 #ifdef ENABLE_DEVELOPING
 	unsigned int list_size;
 	const gnutls_datum_t *cert_list;
 #endif
 
-	if (!tds->login)
+	if (!tds->login) {
+		tdsdump_log(TDS_DBG_ERROR, "No login while checking certificate\n");
 		return GNUTLS_E_CERTIFICATE_ERROR;
+	}
 
 	ret = gnutls_certificate_verify_peers2(session, &status);
 	if (ret < 0) {
@@ -429,7 +457,7 @@ tds_verify_certificate(gnutls_session_t session)
 		}
 		gnutls_x509_crt_init(&cert);
 		gnutls_x509_crt_import(cert, &cert_list[0], GNUTLS_X509_FMT_DER);
-		ret = gnutls_x509_crt_check_hostname(cert, tds_dstr_cstr(&tds->login->server_host_name));
+		ret = gnutls_x509_crt_check_hostname(cert, wanted_certificate_hostname(tds->login));
 		gnutls_x509_crt_deinit(cert);
 		if (!ret) {
 			tdsdump_log(TDS_DBG_ERROR, "Certificate hostname does not match\n");
@@ -441,13 +469,30 @@ tds_verify_certificate(gnutls_session_t session)
 	return 0;
 }
 
+static int
+tds_verify_certificate_tds(gnutls_session_t session)
+{
+	TDSSOCKET *tds = (TDSSOCKET *) gnutls_transport_get_ptr(session);
+
+	return tds_verify_certificate(session, tds);
+}
+
+static int
+tds_verify_certificate_conn(gnutls_session_t session)
+{
+	TDSCONNECTION *conn = (TDSCONNECTION*) gnutls_transport_get_ptr(session);
+
+	return tds_verify_certificate(session, CONN2TDS(conn));
+}
+
 TDSRET
-tds_ssl_init(TDSSOCKET *tds)
+tds_ssl_init(TDSSOCKET *tds, bool full)
 {
 	gnutls_session_t session;
 	gnutls_certificate_credentials_t xcred;
 	int ret;
 	const char *tls_msg;
+	int (*verify_func)(gnutls_session_t session);
 
 	xcred = NULL;
 	session = NULL;	
@@ -478,6 +523,24 @@ tds_ssl_init(TDSSOCKET *tds)
 	if (ret != 0)
 		goto cleanup;
 
+	/* Initialize TLS session */
+	tls_msg = "initializing session";
+	ret = gnutls_init(&session, GNUTLS_CLIENT);
+	if (ret != 0)
+		goto cleanup;
+
+	if (!full) {
+		gnutls_transport_set_ptr(session, tds);
+		gnutls_transport_set_pull_function(session, tds_pull_func_login);
+		gnutls_transport_set_push_function(session, tds_push_func_login);
+		verify_func = tds_verify_certificate_tds;
+	} else {
+		gnutls_transport_set_ptr(session, tds->conn);
+		gnutls_transport_set_pull_function(session, tds_pull_func);
+		gnutls_transport_set_push_function(session, tds_push_func);
+		verify_func = tds_verify_certificate_conn;
+	}
+
 	if (!tds_dstr_isempty(&tds->login->cafile)) {
 		tls_msg = "loading CA file";
 		if (strcasecmp(tds_dstr_cstr(&tds->login->cafile), "system") == 0)
@@ -493,21 +556,11 @@ tds_ssl_init(TDSSOCKET *tds)
 				goto cleanup;
 		}
 #ifdef HAVE_GNUTLS_CERTIFICATE_SET_VERIFY_FUNCTION
-		gnutls_certificate_set_verify_function(xcred, tds_verify_certificate);
+		gnutls_certificate_set_verify_function(xcred, verify_func);
 #endif
 	}
 
-	/* Initialize TLS session */
-	tls_msg = "initializing session";
-	ret = gnutls_init(&session, GNUTLS_CLIENT);
-	if (ret != 0)
-		goto cleanup;
-
-	gnutls_transport_set_ptr(session, tds);
-	gnutls_transport_set_pull_function(session, tds_pull_func_login);
-	gnutls_transport_set_push_function(session, tds_push_func_login);
-
-	/* NOTE: there functions return int however they cannot fail */
+	/* NOTE: these functions return int however they cannot fail */
 
 	/* use default priorities... */
 	gnutls_set_default_priority(session);
@@ -531,6 +584,17 @@ tds_ssl_init(TDSSOCKET *tds)
 	if (ret != 0)
 		goto cleanup;
 
+#ifdef HAVE_GNUTLS_ALPN_SET_PROTOCOLS
+	if (IS_TDS80_PLUS(tds->conn)) {
+		static const unsigned char alpn[] = { TDS8_ALPN_ARRAY };
+		static const gnutls_datum_t tds8_alpn = { (void*) alpn, sizeof(alpn) };
+		gnutls_alpn_set_protocols(session, &tds8_alpn, 1, 0);
+	}
+#endif
+
+	if (full)
+		set_current_tds(tds->conn, tds);
+
 	/* Perform the TLS handshake */
 	tls_msg = "handshake";
 	ret = gnutls_handshake (session);
@@ -539,7 +603,7 @@ tds_ssl_init(TDSSOCKET *tds)
 
 #ifndef HAVE_GNUTLS_CERTIFICATE_SET_VERIFY_FUNCTION
 	if (!tds_dstr_isempty(&tds->login->cafile)) {
-		ret = tds_verify_certificate(session);
+		ret = tds_verify_certificate(session, tds);
 		if (ret != 0)
 			goto cleanup;
 	}
@@ -547,12 +611,16 @@ tds_ssl_init(TDSSOCKET *tds)
 
 	tdsdump_log(TDS_DBG_INFO1, "handshake succeeded!!\n");
 
-	/* some TLS implementations send some sort of paddind at the end, remove it */
-	tds->in_pos = tds->in_len;
+	if (!full) {
+		/* some TLS implementations send some sort of paddind at the end, remove it */
+		tds->in_pos = tds->in_len;
 
-	gnutls_transport_set_ptr(session, tds->conn);
-	gnutls_transport_set_pull_function(session, tds_pull_func);
-	gnutls_transport_set_push_function(session, tds_push_func);
+		gnutls_transport_set_ptr(session, tds->conn);
+		gnutls_transport_set_pull_function(session, tds_pull_func);
+		gnutls_transport_set_push_function(session, tds_push_func);
+	}
+
+	set_current_tds(tds->conn, NULL);
 
 	tds->conn->tls_session = session;
 	tds->conn->tls_credentials = xcred;
@@ -562,6 +630,7 @@ tds_ssl_init(TDSSOCKET *tds)
 cleanup:
 	if (session)
 		gnutls_deinit(session);
+	set_current_tds(tds->conn, NULL);
 	if (xcred)
 		gnutls_certificate_free_credentials(xcred);
 	tdsdump_log(TDS_DBG_ERROR, "%s failed: %s\n", tls_msg, gnutls_strerror (ret));
@@ -582,9 +651,9 @@ tds_ssl_deinit(TDSCONNECTION *conn)
 	conn->encrypt_single_packet = 0;
 }
 
-#else
+#else /* !HAVE_GNUTLS */
 static long
-tds_ssl_ctrl_login(BIO *b, int cmd, long num, void *ptr)
+tds_ssl_ctrl_login(BIO *b TDS_UNUSED, int cmd, long num TDS_UNUSED, void *ptr TDS_UNUSED)
 {
 	switch (cmd) {
 	case BIO_CTRL_FLUSH:
@@ -594,7 +663,7 @@ tds_ssl_ctrl_login(BIO *b, int cmd, long num, void *ptr)
 }
 
 static int
-tds_ssl_free(BIO *a)
+tds_ssl_free(BIO *a TDS_UNUSED)
 {
 	/* nothing to do but required */
 	return 1;
@@ -651,6 +720,7 @@ tds_init_ssl_methods(void)
 	tds_method = meth = BIO_meth_new(BIO_TYPE_MEM, "tds");
 	BIO_meth_set_write(meth, tds_push_func);
 	BIO_meth_set_read(meth, tds_pull_func);
+	BIO_meth_set_ctrl(meth, tds_ssl_ctrl_login);
 	BIO_meth_set_destroy(meth, tds_ssl_free);
 }
 
@@ -668,7 +738,7 @@ tds_deinit_openssl_methods(void)
 static tds_mutex *openssl_locks;
 
 static void
-openssl_locking_callback(int mode, int type, const char *file, int line)
+openssl_locking_callback(int mode, int type, const char *file TDS_UNUSED, int line TDS_UNUSED)
 {
 	if (mode & CRYPTO_LOCK)
 		tds_mutex_lock(&openssl_locks[type]);
@@ -869,13 +939,11 @@ check_alt_names(X509 *cert, const char *hostname)
 		ret = inet_pton(AF_INET, hostname, &ip.v4);
 	}
 	if (ret == 0)
-		return -1;
-
-	ret = -1;
+		ip_size = 0;
 
 	alt_names = X509_get_ext_d2i(cert, NID_subject_alt_name, NULL, NULL);
 	if (!alt_names)
-		return ret;
+		return -1;
 
 	num = sk_GENERAL_NAME_num(alt_names);
 	tdsdump_log(TDS_DBG_INFO1, "Alt names number %d\n", num);
@@ -891,11 +959,9 @@ check_alt_names(X509 *cert, const char *hostname)
 		altlen = (size_t) ASN1_STRING_length(name->d.ia5);
 
 		if (name->type == GEN_DNS && ip_size == 0) {
-			ret = 0;
 			if (!check_name_match(name->d.dNSName, hostname))
 				continue;
 		} else if (name->type == GEN_IPADD && ip_size != 0) {
-			ret = 0;
 			if (altlen != ip_size || memcmp(altptr, &ip, altlen) != 0)
 				continue;
 		} else {
@@ -906,7 +972,7 @@ check_alt_names(X509 *cert, const char *hostname)
 		return 1;
 	}
 	sk_GENERAL_NAME_pop_free(alt_names, GENERAL_NAME_free);
-	return ret;
+	return -1;
 }
 
 static int
@@ -922,12 +988,12 @@ check_hostname(X509 *cert, const char *hostname)
 		return ret;
 
 	/* check by common name (old method) */
-	subject= X509_get_subject_name(cert);
+	subject = X509_get_subject_name(cert);
 	if (!subject)
 		return 0;
 
 	i = -1;
-	while (X509_NAME_get_index_by_NID(subject, NID_commonName, i) >=0)
+	while (X509_NAME_get_index_by_NID(subject, NID_commonName, i) >= 0)
 		i = X509_NAME_get_index_by_NID(subject, NID_commonName, i);
 	if (i < 0)
 		return 0;
@@ -940,7 +1006,7 @@ check_hostname(X509 *cert, const char *hostname)
 }
 
 int
-tds_ssl_init(TDSSOCKET *tds)
+tds_ssl_init(TDSSOCKET *tds, bool full)
 {
 #define DEFAULT_OPENSSL_CTX_OPTIONS (SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_TLSv1)
 #define DEFAULT_OPENSSL_CIPHERS "HIGH:!SSLv2:!aNULL:-DH"
@@ -1001,17 +1067,19 @@ tds_ssl_init(TDSSOCKET *tds)
 		goto cleanup;
 
 	tls_msg = "creating bio";
-	b = BIO_new(tds_method_login);
+	b = BIO_new(full ? tds_method : tds_method_login);
 	if (!b)
 		goto cleanup;
 
-	b2 = BIO_new(tds_method);
-	if (!b2)
-		goto cleanup;
+	if (!full) {
+		b2 = BIO_new(tds_method);
+		if (!b2)
+			goto cleanup;
+	}
 
 	BIO_set_init(b, 1);
-	BIO_set_data(b, tds);
-	BIO_set_conn_hostname(b, tds_dstr_cstr(&tds->login->server_host_name));
+	BIO_set_data(b, full ? (void *) tds->conn : (void *) tds);
+	BIO_set_conn_hostname(b, wanted_certificate_hostname(tds->login));
 	SSL_set_bio(con, b, b);
 	b = NULL;
 
@@ -1029,6 +1097,18 @@ tds_ssl_init(TDSSOCKET *tds)
 	SSL_set_options(con, SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS);
 #endif
 
+#ifdef HAVE_SSL_SET_ALPN_PROTOS
+	if (IS_TDS80_PLUS(tds->conn)) {
+		static const unsigned char tds8_alpn[] = {
+			TDS8_ALPN_ARRAY_LEN, TDS8_ALPN_ARRAY
+		};
+		SSL_set_alpn_protos(con, tds8_alpn, sizeof(tds8_alpn));
+	}
+#endif
+
+	if (full)
+		set_current_tds(tds->conn, tds);
+
 	/* Perform the TLS handshake */
 	tls_msg = "handshake";
 	ERR_clear_error();
@@ -1042,7 +1122,7 @@ tds_ssl_init(TDSSOCKET *tds)
 	}
 
 	/* flush pending data */
-	if (tds->out_pos > 8)
+	if (!full && tds->out_pos > 8)
 		tds_flush_packet(tds);
 
 	/* check certificate hostname */
@@ -1051,19 +1131,23 @@ tds_ssl_init(TDSSOCKET *tds)
 
 		cert =  SSL_get_peer_certificate(con);
 		tls_msg = "checking hostname";
-		if (!cert || !check_hostname(cert, tds_dstr_cstr(&tds->login->server_host_name)))
+		if (!cert || !check_hostname(cert, wanted_certificate_hostname(tds->login)))
 			goto cleanup;
 		X509_free(cert);
 	}
 
 	tdsdump_log(TDS_DBG_INFO1, "handshake succeeded!!\n");
 
-	/* some TLS implementations send some sort of paddind at the end, remove it */
-	tds->in_pos = tds->in_len;
+	if (!full) {
+		/* some TLS implementations send some sort of paddind at the end, remove it */
+		tds->in_pos = tds->in_len;
 
-	BIO_set_init(b2, 1);
-	BIO_set_data(b2, tds->conn);
-	SSL_set_bio(con, b2, b2);
+		BIO_set_init(b2, 1);
+		BIO_set_data(b2, tds->conn);
+		SSL_set_bio(con, b2, b2);
+	}
+
+	set_current_tds(tds->conn, NULL);
 
 	tds->conn->tls_session = con;
 	tds->conn->tls_ctx = ctx;
@@ -1079,6 +1163,7 @@ cleanup:
 		SSL_shutdown(con);
 		SSL_free(con);
 	}
+	set_current_tds(tds->conn, NULL);
 	SSL_CTX_free(ctx);
 	tdsdump_log(TDS_DBG_ERROR, "%s failed\n", tls_msg);
 	return TDS_FAIL;
